/*
    NullBot Robot Client 

    (C) 2005-2011 Jason Hunt

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: nulluser@gmail.com

    FILE: camera.cpp
    
    Network control for dealing with the camera feed.
*/


#include <windows.h>
#include <stdio.h>

#pragma hdrstop

#include "config.h"
#include "main.h"

#include "types.h"

#include "camera.h"
#include "network.h"
#include "camera_win.h"
#include "robot.h"
#include "utility.h"
#include "jpeg.h"
#include "target.h"
#include "profile.h"


#define MODULE_NAME "[Camera] "


/* Camera constructor */
camera_class::camera_class(robot_class *r, int index)
{
    log_header();

    cam_index = index;
    robot = r;
    
    main = r->main;

    flip_horz = 0;
    save_next = 0;


    reset_recv();       // reset packet variables

    d_frame = false;       // display difference frame


    x_size = 0;
    y_size = 0;





    camera_buffer = NULL;
    compressed_image = NULL;

    cam_rx_buffer =   NULL;

    last_frame = NULL;

    processed_frame =  NULL;

   // Need an inital rx buffer for the first size packet 
  cam_rx_buffer =   (unsigned char *) malloc(RX_BUFF);
  

}
/* End of camera_class::camera_class */


/* Camera destructor */
camera_class::~camera_class( void )
{
    log_header();

    free(camera_buffer);
    free(compressed_image);
    free(cam_rx_buffer);

    free(last_frame);
    free(processed_frame);   
}
/* End of camera_class */


























/* One second update function */   
void camera_class::check( void )
{
    if (get_connected())
    {
       // main->alert(MODULE_NAME "Camera %d FPS: %d", cam_index, get_fps());
    }

    if (robot->main->get_current() - cam_last_frame > 5)
    {
        request_image();
    }
        
    if (robot->main->get_current() - cam_last_frame > 10)
    {
        closesocket(server_camera_socket);
        camera_connected = 0;
    }

    if (!camera_connected && !attempting_camera_connect )
        connect_to_camera_server();

    fps = frames;

    frames= 0;
}
/* End of check */
    


/* Draws camera to main's mem_dc */        
void camera_class::draw( void )
{
    HDC memdc = robot->main->get_memdc();

    //SetStretchBltMode(memdc, HALFTONE); //  Looks better
    SetStretchBltMode(memdc, COLORONCOLOR); //   Much faster

    SetBrushOrgEx(memdc, 0, 0, NULL);
    
     
    int x = (int)(x_pos * robot->main->get_display_x());
    int y = (int)(y_pos *  robot->main->get_display_y_act());
                    
    int sx = (int)(x_size * robot->main->get_display_x() );
    int sy = (int)(y_size *  robot->main->get_display_y_act());
        
        
        
        
        
        
   // log(MODULE_NAME " Index: %d    pos1 [%d, %d] pos2 [%d,%d]\n", cam_index, x, y, x+sx, y+sy);
        
        
        
        
    if (flip_horz)
    {
        x += sx;
        sx *= -1;
    }

    //main->get_profile()->start(10);
                
    unsigned int r = StretchDIBits(memdc, x, y, sx, sy, 
                                          0, 0, image_x, image_y,               
                                          camera_buffer, &buffer_info, 
                                          DIB_RGB_COLORS, SRCCOPY);
   // main->get_profile()->stop("Blt");
}
/* End of draw */
    
    
    
/* Save current image in compressed buffer */
void camera_class::save_image( void )
{
    char fname[512];

    char ts[200]; 
    get_time_str(ts);
    
    sprintf(fname, "images\\%s_%d.jpg", ts, cam_index);

    FILE *f = fopen(fname, "wb");
    fwrite(compressed_image, c_size, 1, f);
    fclose(f);
}    
/* End of save_image */    

    
/* Reset all comm variables */
void camera_class::reset_recv(void)
{
    log_header();
 	
    image_x  = 0;
    image_y  = 0;
  
    camera_connected = false;          // true if connected
    attempting_camera_connect = false;
    getting_camera_data = false;

    frames = 0;

    mode = WAIT_SIZE;
    c_index  = 0;
    
    cam_last_frame = 0;  
}
/* End of reset_recv */

    
/* Disconnect from server */
void camera_class::camera_disconnect(void)
{
    log_header();

    closesocket(server_camera_socket);
    camera_connected = 0;
    main->alert(MODULE_NAME "Disconnect.");
}
/* End of disconnect */


/* Send keepalive data */
void camera_class::request_image( void )
{
    if (!camera_connected) return;

    char data_packet[1]; // the data packet
    data_packet[0] = 0x40;

   // alert(MODULE_NAME "Request Image\n");

    if (send(server_camera_socket, data_packet, 1, 0) == SOCKET_ERROR)
    {
        main->alert(MODULE_NAME "Unable to send data");
        camera_connected = 0;
    }
}
/* End of send_control_state */


/* Setup image formate type for blt. */
void camera_class::setup_buffer( void )
{
    log_header();

    memset(&buffer_info, 0, sizeof(buffer_info));

	buffer_info.bmiHeader.biSize = sizeof(buffer_info.bmiHeader);
	buffer_info.bmiHeader.biWidth = image_x;
	buffer_info.bmiHeader.biHeight = image_y;  // - to invert image
	buffer_info.bmiHeader.biPlanes = 1;
	buffer_info.bmiHeader.biBitCount = 24;
	buffer_info.bmiHeader.biCompression = BI_RGB;
	buffer_info.bmiHeader.biSizeImage = image_x * image_y * 3;
	buffer_info.bmiHeader.biClrUsed = 0;
	buffer_info.bmiHeader.biClrImportant = 0;
	
   // update = 0;

    if (camera_buffer) free(camera_buffer);

    camera_buffer = (color_type*) malloc(image_x * image_y * sizeof(color_type));

    if (camera_buffer == NULL)
    {
        log(MODULE_NAME "Unable to get memory for camera_buffer\n");
        return;
    }

    if (compressed_image) free(compressed_image);


    compressed_image = (unsigned char*) malloc(image_x * image_y* sizeof(color_type));
 
    if ( compressed_image == NULL)
    {
        log(MODULE_NAME "Unable to get memory for compressed_image\n");
        return;
    }
 
    if (cam_rx_buffer) free(cam_rx_buffer);
    
 
    cam_rx_buffer =   (unsigned char *) malloc(image_x * image_y* sizeof(color_type));

    if ( cam_rx_buffer == NULL)
    {
        log(MODULE_NAME "Unable to get memory for cam_rx_buffer\n");
        return;
    }

    if (last_frame) free(last_frame);


    last_frame = (color_type*) malloc(image_x * image_y* sizeof(color_type));

    if ( last_frame  == NULL)
    {
        log(MODULE_NAME "Unable to get memory for last_frame \n");
        return;
    }

    if (processed_frame) free(processed_frame);

    processed_frame = (color_type*) malloc(image_x * image_y* sizeof(color_type));

    if ( processed_frame == NULL)
    {
        log(MODULE_NAME "Unable to get memory for processed_frame\n");
        return;
    }
	
	
	
}
/* End of setup_buffer */


/* Deal with size packet */
int camera_class::size_packet(unsigned char *data)
{
    if (data[0] != NC_IMAGE_SIZE)
    {
        main->alert(MODULE_NAME "Invalid image size packet (%x)", data[0]);
        return 1;
    }   
    
    // decode size data
    unsigned int image_x_size = (data[1] << 8) + data[2];
    unsigned int image_y_size = (data[3] << 8) + data[4];

/*
    if (image_x_size > 800 || image_y_size > 600)
    {
        main->alert(MODULE_NAME "Invalid image size (%d)", c_size);
        return 1;
    }
*/

    // resize buffer if needed
    if (image_x_size != image_x || image_y_size != image_y)
    {
        image_x = image_x_size;
        image_y = image_y_size;
    	setup_buffer(); // resize buff for new image size
	}

	// decode compressed size

    integer32 conv;

    conv.bytes.b3 = data[5];
    conv.bytes.b2 = data[6];
    conv.bytes.b1 = data[7];
    conv.bytes.b0 = data[8];

    c_size = conv.n;
            
    c_checksum = (data[9] << 8) + data[10];

            
    if (c_size == 0 || c_size > image_x * image_y* sizeof(color_type))
    {
        main->alert(MODULE_NAME "Invalid image size (%d)", c_size);
        return 1;
    }            
	
	getting_camera_data = true;
                
//    log(MODULE_NAME "Index: %d Size packet done (%d,%d)  len: %d\n", cam_index, image_x, image_y, c_size);

	return 0;
}
/* End of size_packet */


/* Compute 16 bit checksum */
static inline unsigned short get_checksum16(unsigned char *buff, unsigned int len)
{
    unsigned int *p = (unsigned int *) buff;
    unsigned int i;
    unsigned int check = 0;

    for (i = 0; i < len / 4; i++)
        check += *(p + i);

    return((check >> 24) | check);
}
/* End of get_checksum */



/* Deal with image packet */
void camera_class::image_packet( void )
{
    log_header();

    // Make sure word boundry is zero for checksum function

    compressed_image[c_size + 0] = 0;
    compressed_image[c_size + 1] = 0;
    compressed_image[c_size + 2] = 0;
    compressed_image[c_size + 3] = 0;

    unsigned short checksum = get_checksum16(compressed_image, c_size);

    request_image(); // request next image

    if (checksum!= c_checksum)
    {
        main->alert(MODULE_NAME "Image checksum fail. expected: %d  actual: %d  (%d)\n", 
        c_checksum, checksum,  checksum -  c_checksum);
        return;
    }
    
    // Save image as requested
    if (save_next)
    {
        save_image();
        save_next = 0;
    }
    
  // log(MODULE_NAME "Index: %d c_size: %d\n", cam_index, c_size);
    
    
    decompress_image(compressed_image, &c_size, camera_buffer);

	// process image if targeting is enabled
	if (robot->get_targetting())  process_frame();    
                 
    cam_last_frame = robot->main->get_current();
                 
	frames++;   // increment frame counter

    draw();	// force display
}
/* End with image packet */


/* Get data from the image server */
void camera_class::read_camera_data(void)
{
    int n_bytes = 0;
    
//    log("Read camera\n");    

    while (1)
    {
        // Waiting for image size packet 
        if (mode == WAIT_SIZE)
        {
            n_bytes = recv(server_camera_socket, 
                           (char *)(cam_rx_buffer + c_index), 
                           SIZE_LEN - c_index, 0);
        
            if (n_bytes <= 0) break;                    // No more data
                
            c_index += n_bytes;
                    
            if (c_index < SIZE_LEN) continue;
            
                                                    
            // We have the entire packet 
//          if (c_index == SIZE_LEN)
            {
                if (size_packet(cam_rx_buffer) == 0)
                    mode = WAIT_DATA;                   // Wait for data packet

                c_index = 0;
            }                                  
        }
        else
        // Waiting for image packet
        {    
            // We know what size to expect, load directly into compressed image buffer
            n_bytes = recv(server_camera_socket, 
                           (char *)(compressed_image + c_index), 
                           c_size - c_index, 0);


            if (n_bytes <= 0) break;                    // No more data

            c_index +=  n_bytes;

            if (c_index < c_size) continue;


            // We have the entire packet 
//            if (c_index == c_size)
            {
                mode = WAIT_SIZE;       // Wait for next size packet
                
                image_packet();
                c_index = 0;
            }
        }
    }
}
/* End of read_camera_data */



/* Deal with network messages */
void camera_class::camera_socket_message(WPARAM wParam,LPARAM lParam)
{
    switch (WSAGETSELECTEVENT(lParam))
    {
        // data back from server
        case FD_READ:
            read_camera_data();
            break;

//        case FD_WRITE:
//            break;
                        
//        case FD_ACCEPT:
//            break;

        case FD_CONNECT:
            if(WSAGETSELECTERROR(lParam))
            {
                reset_recv();
                main->alert(MODULE_NAME "Time out.");
                attempting_camera_connect = false;
                closesocket(server_camera_socket);
            }
            else
            {
                reset_recv();
                main->alert(MODULE_NAME "Connected.");
                camera_connected = true;
                attempting_camera_connect = false;
                request_image(); // start the request chain
            }
            break;

        case FD_CLOSE:
            reset_recv();
            main->alert(MODULE_NAME "Server disconnect.");
            camera_connected = false;
            closesocket(server_camera_socket);
            break;


    }
}
/* End of network_message */


/* Attept to connect to te server */
void camera_class::connect_to_camera_server( void )
{
    main->alert(MODULE_NAME "Connecting to %s:%d", 
                                robot->get_host_name(),
                                robot->get_base_port() + 1 + cam_index);
                                

    // create the socket
    server_camera_socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

    if (server_camera_socket == INVALID_SOCKET)
    {
        main->alert(MODULE_NAME "Socket error.");
        return;
    }

    LPHOSTENT hostent;
    hostent = gethostbyname(robot->get_host_name());

    if (!hostent)
    {
        main->alert(MODULE_NAME "Host name error.");
        return;
    }

    SOCKADDR_IN server;

    memset(&server, 0, sizeof(server));

    server.sin_addr = *((LPIN_ADDR)*hostent->h_addr_list);
    server.sin_port = htons(robot->get_base_port() + 1 + cam_index);
    server.sin_family = AF_INET;

    WSAAsyncSelect(server_camera_socket, robot->main->get_hwnd(),
                   WM_CAMEVENT + cam_index, FD_WRITE | FD_CONNECT | FD_READ | FD_CLOSE);

    camera_connected = false;
    attempting_camera_connect = true;

    connect(server_camera_socket, (LPSOCKADDR)&server, sizeof(struct sockaddr));
}
/* End of connect_to_server */



/* Draw a box */
void camera_class::draw_box(unsigned int x1, unsigned int y1, 
                            unsigned int x2, unsigned int y2, 
                            color_type *m)
{

    CLAMP(x1, 0, image_x);
    CLAMP(x2, 0, image_y);
    CLAMP(y1, 0, image_x);
    CLAMP(y2, 0, image_y);

            
    for (unsigned int y = y1; y < y2 ; y++) 
    {
        unsigned int index = x1 + y * image_x;

        for (unsigned int x = x1; x < x2 ; x++)
        {
            m[index].r = 255;
            m[index].g = 0;
            m[index].b = 0;
            index++;
        }
    }
}
/* end of draw box */





