/*
    NullBot Robot Client 

    (C) 2005-2011 Jason Hunt

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: nulluser@gmail.com

    FILE: robot_camera.h
*/


#ifndef CAMERA_H
#define CAMERA_H

#define WM_CAMEVENT (WM_USER+2)

//#define MAX_COMPRESSED_SIZE 1024*768*3

//#define REC_BUFFER_SIZE     (MAX_COMPRESSED_SIZE)

#define RX_BUFF      1000

#define WAIT_SIZE   0
#define WAIT_DATA   1

#define SIZE_LEN 11
#define NC_IMAGE_SIZE   0x40


class camera_class
{
  public:
    camera_class(robot_class *r, int index);
    ~camera_class();
    
    void reset_recv( void );





    void draw( void );

    void check( void );
    void camera_disconnect(void);
    void request_image( void );
    int size_packet(unsigned char *data);
    void image_packet( void );
    int camera_process_packet(unsigned char *data, unsigned int len);
    void read_camera_data(void);
    void camera_socket_message(WPARAM wParam, LPARAM lParam);
    void connect_to_camera_server( void );
    void camera_start( void );

    void setup_buffer( void );

    void save_image( void );
    


    // Draw box on frame
    void draw_box(unsigned int x1, unsigned int y1, 
                  unsigned int x2, unsigned int y2, 
                  color_type *m);
              
    // Target tracking
    void process_frame( void );
    unsigned char get_change(unsigned int x, unsigned int y);


    int get_fps( void ) { return fps; }
    int get_connected( void ) { return camera_connected; }


          
            
    void set_pos(float xp, float yp) { x_pos = xp; y_pos = yp; };
    void set_size(float xs, float ys) { x_size = xs; y_size = ys; };
    void set_flip_horz(char f) { flip_horz = f; };              
    
    void set_save_next(void) { save_next = 1;};
                  
  private:

    // buffer for image processing
    color_type *last_frame;
    color_type *processed_frame;

    SOCKET server_camera_socket;

    bool camera_connected ;                     // true if connected
    bool attempting_camera_connect ;
    bool getting_camera_data ;

    unsigned int frames ;    
    unsigned int fps;
        
    
    unsigned int image_process;                         
 
    double cam_last_frame ;
    
    bool d_frame;                                           // display difference frame

   // unsigned int update;    // used to keep track of camera sweep


    BITMAPINFO buffer_info;

    color_type *camera_buffer;                  // stores image from camrea

    int cam_index;                                          // index of camera
    unsigned int image_x;
    unsigned int image_y;

    unsigned int mode ;                                     // COmm mode
    unsigned int c_size;                        // Comm index, packet remaining, size
    unsigned short c_checksum;                              // packet checksum
    unsigned int c_index ;                              // cammera buffer index
  

    unsigned char *compressed_image;//
    unsigned char *cam_rx_buffer;

    bool save_next;

    
    char flip_horz;
    
    // Image display position
    float x_pos;
    float y_pos;
    
    // Image display size
    float x_size;
    float y_size;       

    main_class *main;                                       // Link to main
    robot_class *robot;
   
};


#endif


