/*
    NullBot Robot Client 

    (C) 2005-2011 Jason Hunt

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: nulluser@gmail.com
*/

#include <windows.h>
#include <stdio.h>
#include <string.h>

#include "types.h"
#include "utility.h"
#include "config.h"


#define MODULE_NAME  "[Config] "


/* Show config enteries */
void show_config( config_entry *config )
{
    config_entry * c = config;
 
    log(MODULE_NAME "Show Config\n");
 
    while(c)
    {
        log(MODULE_NAME "[%s]\n", c->data);

        config_entry * l = c->next_line;

        while(l)
        {
            log(MODULE_NAME "  %s\n", l->data);
            l = l->next_line;
        }    
 
        c=c->next;
     }

    log(MODULE_NAME "End Show Config\n");
}
/* End of show_config */



/* Return first config entry for section name */
config_entry *  config_get_line( config_entry *config, char *section )
{
    config_entry * c = config;

    while(c)
    {
        if (!strcmp(c->data, section)) 
            return (c->next_line);

        c=c->next;
     }

    return (NULL);
}
/* End of config_get_line */




/* Returns true if whitespace */
int is_white(char c)
{
    return (c == ' ') || (c == '\t');
}
/* End of is_white */


/* Returns true if newline */
int is_newline(char c)
{
    return (c == 0x0d) || (c == 0x0a);
}
/* End of is_white */



/* Gets the nextr string in a line, up to a space or endline */
void get_next_string(char *line, unsigned int &i, char *str)
{
    int done_str = 0;   // flag for done with string
    
    unsigned int s = 0; // index into target string

    // eat white space at front of line
    while (is_white(line[i]) && i < CONFIG_LINE_MAX && line[i]) i++;
  
    while (!done_str && i < CONFIG_LINE_MAX)
    {
        char ch = line[i++];
                               
        if (ch == ' ' || ch == '\t' ||  ch == 0)
            done_str = 1;
        else
            str[s++] = ch;
    }

    str[s] = 0;   // null terminate
}
/* End of get_next_string */



/* Loads configuration values from config file */
void load_config( config_entry **config, char *config_filename )
{
    FILE *f = fopen(config_filename, "rt");
    
    if (!f)
    {
        log(MODULE_NAME "Unable to load config file\n");
        return;
    }    


    config_entry *cur_section = NULL;
    config_entry *line_tail = NULL;
    config_entry *section_tail = NULL;


    // load each line
    while (!feof(f))
    {
        char line[CONFIG_LINE_MAX];    // temperary line

        unsigned int i = 0;     // index
        
        int line_done = 0;      // line end
        
        // read line
        while (i < CONFIG_LINE_MAX && !line_done)
        {
            char ch = fgetc(f);             // get char from file

            // eat rest of line
            if (ch == '#')
            {
                while (ch != 0x0a && ch != 0x0d && !feof(f))  ch = fgetc(f);
            } else    
    
            // limit check
            if (ch == 0x0d || ch == 0x0a || feof(f)) line_done = 1; else
                line[i++] = ch;
        }            
               
        line[i] = 0; // null terminate
        
        // check for blank line
       // if (i > 0)
       //     parse_func(param, line);
            
            
        if (i > 0)
        {  
            config_entry *c = (config_entry *)malloc(sizeof(config_entry));
            c->next = NULL;
            c->next_line = NULL;
            c->data = (char *)  malloc(strlen(line));
            c->data[0] = 0;

            if (*config == NULL)
            {
                *config = c;
                section_tail = c;
                line_tail = c;
            }
                
            log ("line (%s)\n", line);
                
                              
            if (line[0] == '[' && line[strlen(line)-1] == ']')
            {
                section_tail->next = c;

                section_tail = c;
                line_tail = c;

                line[strlen(line) -1] = 0;
           
                strcpy(c->data, line+1);

                cur_section = c;
            
            }
            else
            {
                if (cur_section == NULL)
                {
                    log("Config line not in section: %s\n", line);
                }
                else
                {
                    line_tail->next_line = c;
                    line_tail = c;
                    strcpy(c->data, line);
                }
            }
            
        }
            
           
           // parse_line(line);
           
    }    

    show_config(*config);

    
    log(MODULE_NAME "Loaded config %s\n", config_filename);
   
    
    fclose(f);
}
/* End of load config */



