/*
    NullBot Robot Client 

    (C) 2005-2011 Jason Hunt

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: nulluser@gmail.com

    FILE: robot_comm.h
*/

#ifndef NETWORK_H
#define NETWORK_H



#define WM_NETEVENT (WM_USER+1)

#define COMM_BUFF_SIZE   512

#define COMM_PACKET_WAIT    0 
#define COMM_PACKET_DATA    1
#define COMM_PACKET_FLUSH   2

#define COMM_RX_MAX         1024
#define COMM_RX_MIN         3




class network_class 
{
  public:

    network_class(robot_class *r);
    ~network_class();

    void network_error ( void );
    void check( void );
    
    void update ( void );


    void set_host_name(char *name);
    void connect_to_server( void );
    void shutdown_network(void);
    void echo_controller_value(unsigned char *comm);

    int validate_packet( unsigned char *bf, unsigned int len );
    void robot_process_telemetry(unsigned char *bf, unsigned int len);

    int process_packet( unsigned char *bf, unsigned int len );
    int node_process_byte( unsigned char b );
    void read_server_data(void);

    void send_string( char *str );
    void send_version( void );
    void send_packet(unsigned char *packet, unsigned int size);

    void fire_on(void);
    void fire_off(void);

    void send_controller_values();
    void send_keep_alive( void );

    void remote_shutdown( void );
    void do_connect(void);
    void control_socket_message(WPARAM wParam,LPARAM lParam);
    void disconnect(void);
    void get_servername( void );

    void motion_stop ( void );

    bool get_connected( void ) { return connected; };
    
    bool set_noconnect( bool n ) { no_connect = n; };
     

  private:

    SOCKET server_control_socket;

    bool connected ;          // true if data connected
    bool attempting_connect ;
    bool no_connect ;         // true if we do not want to connect
 
    double last_update;

    unsigned int comm_state;
    unsigned int b_index;
    unsigned int packet_size ;
    unsigned char rx_buff[COMM_RX_MAX];

    robot_class * robot;
    main_class * main;              
};

#endif










