/*
    NullBot Robot Client 

    (C) 2005-2011 Jason Hunt

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: nulluser@gmail.com
    
    Robot methods
*/


#include <math.h>
#include <windows.h>
#include <stdio.h>

#include "config.h"
#include "main.h"
#include "types.h"
#include "automode.h"

#include "robot.h"

#include "utility.h"
#include "config.h"
#include "main.h"
#include "collision.h"
#include "camera_win.h"

#define MODULE_NAME "[Robot] "


/* Robot constructor */
robot_class::robot_class( main_class *m )

{
    log_header();
        
    main = m;

    // defaults
    base_port = 1200;
    strcpy(name, "Unknown");
    strcpy(host_name, "127.0.0.1");

    num_cameras = 0;

   // load_config("nullbot.conf", robot_parse_wrapper, this);


    parse_config();
    

    comm = new network_class(this) ;
    
    wifi_signal = 0;
    
    target_speed = 0;
    target_ddir = 0;
    
    
    accel_1 = 0.015;
    accel_2 = 0.030;
    

    speed = 0;
    ddir = 0;


    motion(0, 0,0);

    // Control data
    pan = 0x80; 
    tilt = 0x80;

    // Telemetry
    z_angle = 0;
    x_vel = 0;
    y_vel = 0;
    wifi_signal = 0;   
    ir_dist = 0;
    
    targetting = 0;
    

    auto_mode = 0;
master_speed = 0;

    
    
    last_update = main->get_current();

              
    set_auto_mode(0);

    
    check();
    update();
 
 
    main->alert("Robot %s Loaded. Host: %s Port: %d", name, host_name, base_port);
 
}
/* End of robot constructor */



/* Robot destructor */
robot_class::~robot_class( void )
{
    log_header();


    delete comm;    

    for (int i = 0; i < num_cameras; i++)
        delete cameras[i];

}
/* End of robot destructor */







/* Parse config enteries for this module */
void robot_class::parse_config( void )
{
    log_header();

//    log(MODULE_NAME "Parse: %s\n", line);


log("Robot parse config\n");

       unsigned int index = 0;
        char str[CONFIG_LINE_MAX];




    config_entry *line = config_get_line(main->get_config(), "robot");
    
    while (line)
    {
        log("Data (%s)\n", line->data);
        
        index = 0;


        get_next_string(line->data, index, str);
        
        log("next str: (%s) %d\n", str, strlen(str));
        
        if (!strcmp(str, "robot"))
        {
            get_next_string(line->data, index, name);

            get_next_string(line->data, index, host_name);

            get_next_string(line->data, index, str);
        
            base_port = atoi(str);
        
            get_next_string(line->data, index, str);
        
            auto_mode= atoi(str);
            
            log("Host: %s\n", host_name);
        }
               

        if (!strcmp(str, "camera"))
        {
            char tmp_name[256];
            get_next_string(line->data, index, tmp_name);

            get_next_string(line->data, index, str);
            float x_pos = atof(str) / 100.0;
        
            get_next_string(line->data, index, str);
            float y_pos = atof(str) / 100.0;

            get_next_string(line->data, index, str);
            float x_size = atof(str) / 100.0;

            get_next_string(line->data, index, str);
            float y_size = atof(str) / 100.0;

            get_next_string(line->data, index, str);
            char flip_horz = atoi(str);

            log(MODULE_NAME "Camera load %s\n",  tmp_name);

            cameras[num_cameras] = new  camera_class(this, num_cameras);        
        
            cameras[num_cameras]->set_pos(x_pos, y_pos);
            cameras[num_cameras]->set_size(x_size, y_size);
            cameras[num_cameras]->set_flip_horz(flip_horz);
                    
            num_cameras++;
        }

        line = line->next_line;
    }
}
/* End of parse config */

















/* Update all values for the aminal */
void robot_class::update(void)
{
    log_header();



    // Compute speed


    update_motion();



    if (auto_mode)
    {
        robot_automode();
    }

    double dt = main->get_current() - last_update;

    last_update += dt;

    comm->update();
}
/* End of robot update */




/* One second update function */ 
void robot_class::check( void )
{
    log_header();

    for (int i = 0; i < num_cameras; i++)
    {
        cameras[i]->check();
    }    

    comm->check();                             
}
/* End of robot_class::check */




/* Save all camera images */
void robot_class::save_images( void )
{
    for (int i = 0; i < num_cameras; i++)
        cameras[i]->set_save_next();
}
/* End of save images */






/* Get control data for robot */
int robot_class ::get_control_data(unsigned char *buff)
{
    unsigned char i = 0;

    if (buff != NULL)
    {
        buff[i++] = servo_0;
        buff[i++] = servo_1;
        buff[i++] = servo_2;
        buff[i++] = servo_3;
    }
        
    return i;
}
/* End of get_control_data */


/* Process robot telemetry */
void robot_class::process_telemetry(unsigned char *bf, unsigned int len)
{
//    main->alert("Got telemetry");

    //ir_dist = (float)(bf[4] + bf[5]*256)*360/65536.0;
//    z_angle = (float)(bf[6] + bf[7]*256)*360/65536.0;
//    x_vel = (float)(bf[8] + bf[9]*256)/65536.0   * 10;
//    y_vel = (float)(bf[10] + bf[11]*256)/65536.0   * 10;    
  
    wifi_signal = bf[20] * 100 / 255.0;
    
    int i = 21;
    
    
    float_32 t;
    
    t.b0 = bf[i++];
    t.b1 = bf[i++];
    t.b2 = bf[i++];
    t.b3 = bf[i++];
        
    roll = t.f;
    
    t.b0 = bf[i++];
    t.b1 = bf[i++];
    t.b2 = bf[i++];
    t.b3 = bf[i++];
        
    pitch = t.f;
    
    
    t.b0 = bf[i++];
    t.b1 = bf[i++];
    t.b2 = bf[i++];
    t.b3 = bf[i++];
        
    yaw = t.f;
    
 // main->alert("l: %d  Roll %+7.2f Pitch %+7.2f Yaw %+7.2f ",  len,roll, pitch, yaw);
    
    
/*
    main->alert("Wifi Signal: %4.1f  Angle: %4.1f Velx: %4.1f  Vely: %4.1f",  
                       wifi_signal,
                         z_angle,
                         x_vel,
                         y_vel
                         
                         );*/



}
/* End of process_telemetry */


/* Create telemetry string */
void robot_class::telemetry_string(char *st)
{
    sprintf((char *)st, "FPS: %d  Wifi Signal:%4.1f  IR: %4.1f Angle:%4.1f  Velx:%4.1f  Vely:%4.1f",  
                        cameras[0]->get_fps(), wifi_signal,
                        ir_dist, z_angle, x_vel,  y_vel);
}
/* End of telemetry string */


/* Manually Set motions for the first anaimal */
void robot_class::motion(double mspeed, double s, double d)
{
    log_header();


master_speed = mspeed;


//    log(MODULE_NAME "motion %4.1f  %4.1f\n", s, d);
    

    if (s < -1) s = -1;
    if (s >  1) s =  1;
    
    if (d < -1) d = -1;
    if (d >  1) d =  1;
    

    target_speed = s;

    target_ddir = d * master_speed;// * 0.04;

//    l_motor = l;
//    r_motor = r;

    update();

}
/* End of animal_motion */



/* Update all values for the aminal */
void robot_class::update_motion(void)
{
    log_header();

    double accel;


    if (fabs(target_speed) > 0.001)
        accel = accel_1*dt*300;                // Accelerating
    else
        accel = accel_2*dt*300;                // Decelerating



    speed = (accel) * target_speed + (1-accel) * speed;


    ddir = (accel) * target_ddir + (1-accel) * ddir;



    //if (fabs(speed) < 0.1) speed = 0;
    

//     main->alert(MODULE_NAME "motion %4.4f  dir: %4.4f", speed, ddir);


    double amp1 = 100;
    double amp2 = 128;



    double servo_0_temp;
    double servo_1_temp;
    double servo_2_temp;
    double servo_3_temp;

    servo_0_temp = 128 + speed * amp1;
    servo_1_temp = 128 + ddir * amp2;
    servo_2_temp = 128 - ddir * amp2;
    servo_3_temp = 0;
            
        
        
//      robot_win->alert("rm: %4.3f   lm: %4.3f ", rm, lm);
       
    //  robot_win->alert("rm_temp: %4.3f   lm_temp %4.3f ", lm * amp , rm * amp );
        
        
        
    if (servo_0_temp < 0) servo_0_temp = 0;
    if (servo_0_temp > 255) servo_0_temp = 255;
        
    if (servo_1_temp < 0) servo_1_temp = 0;
    if (servo_1_temp > 255) servo_1_temp = 255;

    if (servo_2_temp < 0) servo_2_temp = 0;
    if (servo_2_temp > 255) servo_2_temp = 255;

    if (servo_3_temp < 0) servo_3_temp = 0;
    if (servo_3_temp > 255) servo_3_temp = 255;



    //  robot_win->alert("  lm_temp %4.3f rm_temp: %4.3f  ", lm_temp, rm_temp);


    servo_0 = (unsigned char) servo_0_temp;
    servo_1 = (unsigned char) servo_1_temp;
    servo_2 = (unsigned char) servo_2_temp;
    servo_3 = (unsigned char) servo_3_temp;

}




/* Move robot camera pan */
void robot_class ::move_pan( int v )
{
    log_header();

    if (v > 0)
    {
        if (pan + v < 255) pan += v;
    }

    if (v < 0)
    {
        if (pan + v > 0) pan += v;
   
    }
}
/* End of move_pan */



/* Set robot camera pan */
void robot_class ::set_pan( int v )
{
    if (v < 0) v = 0;
    if (v > 255) v = 255;

    pan = v;

}
/* End of set_pan */


/* Set robot camera tilt */
void robot_class ::set_tilt( int v )
{
    if (v < 0) v = 0;
    if (v > 255) v = 255;

    tilt = v;
}
/* End of set tilt */



void robot_class::toggle_automode( void ) 
{ 
    auto_mode = !auto_mode; 

    if (auto_mode == 0)
        stop_motion();
};


void robot_class ::stop_motion( )
{
    log_header();

    motion(0,0, 0);
}


/* Pass network message along to camera by index */
void robot_class::camera_message(int index, WPARAM wparam, LPARAM lparam)
{
    log_header();

    cameras[index]->camera_socket_message(wparam, lparam); 
}
/* End of camera_message */





double robot_class::get_average_fps( void ) 
{
    double average = 0;
    
    int num_connected = 0;
            
    
    for (int i = 0; i < num_cameras; i++)
    {
    
        if (cameras[i]->get_connected())
        {
            num_connected++;
        
            average += cameras[i]->get_fps();
        }
    }
    
    if (num_connected == 0) return 0;
    
    
    return average /(double) num_connected;
}




/* 3D engine for IUM display */

/* Rotation about origin */
inline void rotate( float a, float x1, float y1, float  *nx, float *ny )
{
    a *= M_PI/180.0;
    
	*nx = x1 * cos(a) - y1 * sin(a);        
	*ny = x1 * sin(a) + y1 * cos(a);
}
/* End of rotate */



/* Draw face */
void robot_class::draw_face(HDC dc, face f,  point *p)
{
    int t_p0 = f.l0;
    int t_p1 = f.l1;
    int t_p2 = f.l2;

		if ((p[t_p0].sy * (p[t_p2].sx - p[t_p1].sx) + 
		     p[t_p1].sy * (p[t_p0].sx - p[t_p2].sx) + 
		     p[t_p2].sy * (p[t_p1].sx - p[t_p0].sx)) < 0) return;


    HBRUSH b = CreateSolidBrush(f.color);
    HPEN p1 = CreatePen(PS_SOLID, 1, RGB(10,10,10));

    
    SelectObject(dc, b);
    SelectObject(dc, p1);

    POINT points[4];

    points[0].x = (int)p[f.l0].sx; 
    points[0].y = (int)p[f.l0].sy;

    points[1].x = (int)p[f.l1].sx; 
    points[1].y = (int)p[f.l1].sy;

    points[2].x = (int)p[f.l2].sx; 
    points[2].y = (int)p[f.l2].sy;

    points[3].x = (int)p[f.l3].sx; 
    points[3].y = (int)p[f.l3].sy;


    Polygon(dc, points, 4);
    
    DeleteObject(b);
    DeleteObject(p1);
    
/*


    draw_line(dc, l[f.l0], p);
    draw_line(dc, l[f.l1], p);
    draw_line(dc, l[f.l2], p);
    draw_line(dc, l[f.l3], p);
    
    
    
  */  
    
}
/* Enf of draw face */

/* Draw IMU data */
void robot_class::draw_imu(HDC dc)
{
    HPEN p = CreatePen(PS_SOLID, 1,RGB(255, 255,255));
    HBRUSH b = CreateSolidBrush(RGB(255, 255,255));
    
    
    SelectObject(dc, p);
    SelectObject(dc, b);
    
    Rectangle(dc, 0, 0, main->get_display_x(), main->get_display_y());
    DeleteObject(p);
    DeleteObject(b);
    

    char buff[255];
    sprintf(buff, "Roll %+7.2f  Pitch %+7.2f  Yaw %+7.2f ", roll, pitch, yaw);
    TextOut(dc, 1, 20, buff, strlen(buff)); 

    point points[] = {{-1,1,-1}, {1,1,-1}, {1,-1,-1}, {-1,-1,-1},                            
                       {-1,1,1},  {1,1,1}, {1,-1,1}, {-1,-1,1}};

                     
    // hack for back face culling
    face faces[] = {{0, 1, 2, 3, RGB(0x00, 0x66, 0xff)}, {7, 6, 5, 4, RGB(0x00, 0x66, 0xff)},
                    {1, 5, 6, 2, RGB(0x33, 0xcc, 0x33)}, {4, 0, 3, 7, RGB(0x33, 0xcc, 0x33)},
                    {1, 0, 4, 5, RGB(0xff, 0x19, 0x19)}, {3, 2, 6, 7, RGB(0xff, 0x19, 0x19)}  };                     
                                          
                     
                     
    int dx = main->get_display_x();     // Get screen dims
    int dy = main->get_display_y();           
    float s = 2*(dx+dy)/2.0;;             
    
    int num_points = sizeof(points) / sizeof(point);
    int num_faces = sizeof(faces) / sizeof(face);

    double yaw1 = yaw - 143;        // Correct yaw

    // Rotate and project
    for (int i = 0; i < num_points; i++)
    {
        // Rotate points
		rotate(-roll,  points[i].x,  points[i].y,  &points[i].rx, &points[i].ry);
		rotate(-pitch,  points[i].z,  points[i].ry,  &points[i].rz, &points[i].ry);
		rotate(yaw1,    points[i].rx,   points[i].rz,  &points[i].rx, &points[i].rz);
		
		// Project to screen
	    points[i].sx = dx/2.0 + s*(points[i].rx / (points[i].rz + 10.0));
        points[i].sy = dy/2.0 - s*(points[i].ry / (points[i].rz + 10.0));
    }


    // Draw Faces
    for (int i = 0; i < num_faces; i++)
        draw_face(dc, faces[i], points);


}
/* End of draw IMU */





















