/*
    NullBot Robot Client 

    (C) 2005-2011 Jason Hunt

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: nulluser@gmail.com

    FILE: utility.cpp
*/


#include <windows.h>
#include <stdio.h>
#include <math.h>

#include "types.h"




#include "utility.h"
#include "time.h"
#include "config.h"
#include "main.h"

#define MODULE_NAME "[Utility] "







/* Return current time in seconds */
double get_time( void )
{
    static LARGE_INTEGER freq;
    bool init = 0;

    if (!init)
    {
        QueryPerformanceFrequency(&freq);
        init = 1;
    }

    LARGE_INTEGER c;

    QueryPerformanceCounter(&c);

    double value = c.QuadPart / (double)freq.QuadPart;
   
    return(value);
}
/* End of get_time */



void get_time_str(char *st1)
{

    SYSTEMTIME st;
    GetLocalTime(&st); 


    
    sprintf(st1, "%02d %02d %02d - %02d %02d %02d",st.wMonth, st.wDay, st.wYear, 
                                             st.wHour, st.wMinute, st.wSecond);

}




/* Write formatted data to the debugging console */
void log(char *fmt, ... )
{
    static bool log_init = false;
    static FILE *log_file = NULL;

    if (!log_init)
    {
        log_file = fopen("log.txt", "wt");
        log_init = true;
     }

   // static double start_time = get_time();


    const unsigned int max = 30000;

    va_list args;
    char text[max];

    va_start(args, fmt);
    vsnprintf(text, max, fmt, args);
    fprintf(log_file, "%s", text);

    va_end(args);    
 
   // fprintf(log_file, "\n", text);
    
    fflush(log_file);
}
/* End of log */



/* Write formatted data to the debugging console */
void console(char *fmt, ... )
{
    static HANDLE h_console = INVALID_HANDLE_VALUE;

    if (h_console == INVALID_HANDLE_VALUE)
    {
        AllocConsole();
        h_console =  GetStdHandle(STD_OUTPUT_HANDLE);
     }

    const unsigned int max = 1024;

    va_list args;
    char text[max];

    va_start(args, fmt);
    vsnprintf(text, max, fmt, args);
    DWORD to_screen = 0;
    WriteConsole(h_console, text, strlen(text), &to_screen, NULL);

    va_end(args);    
}
/* End of console */



/* Stop log */
/*void main_class::log_stop( void )
{
 if(log_file !=NULL)
    fclose(log_file);
}*/
/* End of log-stop */


/* Show a buffer */   
void show_buffer(char *header, unsigned char *data, int len)
{
    char item[5];
     
    unsigned int max = len * 5 + 100;

    
    char buff[max];
   // char item[10];
    
    
    buff[0] = 0;
    
    
    
    for (unsigned int i = 0; i < len; i++)
    {
        sprintf(item, "%2x ", data[i]);
        
        strcat(buff, item);
    }

    //alert("%s Len %d - %s", header, len, buff);
 
}    
/* End of show-buffer */
    
    
    
/* Return a random value in [s..e] */
double f_rand(double s, double e)
{
  double val = s + (rand() / (double) RAND_MAX) * (e - s);

   // if (val == 0) return(f_rand(s, e));

    return(val);
}
/* End of f_rand */


/* Wrap an angle around the unit circle */
double angle_wrap(double a)
{
    if (a > 2*M_PI ) return(a - 2*M_PI);

    if (a < 0 ) return(a + 2*M_PI);    

    return(a);
}
/* End of angle_wrap */



/* Return size of vector */
double vect_mag(double x, double y)
{
    return(sqrt(x*x+y*y));
}
/* End of vect mag */



/* accessor functons for logging state */
void set_logging(bool new_state)
{
  //  logging_enabled = new_state;
}
/* end of set_logging */


/* accessor functons for logging state */
bool get_logging( void )
{
  //  return(logging_enabled);
}
/* end of set_logging */


/* adds an item to the log file */
void addlog(char *s)
{
   SYSTEMTIME st;
    GetLocalTime(&st); 

    FILE *f = fopen("log.txt", "at");   

    if (!f)
    {
        //alert("Unable to create log file");
        return;
    }
    
    fprintf(f, "[%d/%d/%d - %d:%d:%d%] %s\n",st.wMonth, st.wDay, st.wYear, 
                                             st.wHour, st.wMinute, st.wSecond, s);

    fclose(f);  // this is slow, but we want whatever data we have if the client crashes
}
/* end of log */



/* Clamp a value */
void clamp ( double &val, double max )
{
    if (val > 0)
    {
        if (val > max) val = max;
    } else
    {
        if (val < -max) val = -max;       
    }
}
/* End of clamp */


/* Compute 16 bit checksum */
unsigned char get_checksum(unsigned char *buff, unsigned int len)
{
    unsigned int i;
    unsigned char check = 0;

    for (i = 0; i < len; i++)
        check += buff[i];

    return(check);
}
/* End of get_checksum */


/* Seed the randon number generator */
void rand_seed( void )
{

    srand(time(NULL));

}
/* End of seed rand */










    
