/*
    NullBot Robot Platform - Server - 2005-2011 Jason Hunt
    
    GNU GENERAL PUBLIC LICENSE Version 2, June 1991
        
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: nulluser@gmail.com
    
    File: camera.c
*/


#include "camera.h"
#include "camera_drv.h"
#include "utility.h"
#include "server.h"
#include "../netcomm.h"


#include <linux/videodev2.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <signal.h>

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <pthread.h>
#include <memory.h>
#include <errno.h>

#define REC_BUFFER  1024

#define MODULE_NAME "[Camera] "


int camera_delay = 100000;
int save_image = 0;

extern int transmit;



/* Compute 16 bit checksum */
static inline unsigned short get_checksum16(unsigned char *buff, unsigned int len)
{
    unsigned int *p = (unsigned int *) buff;
    unsigned int i;
    unsigned int check = 0;

    for (i = 0; i < len / 4; i++)
        check += *(p + i);

    return((check >> 24) | check);
}
/* End of get_checksum16 */



/* Send camera image */
int send_camera_image( camera_type *camera, int client_sd )
{
    unsigned int out_size;

    cameradrv_frame (camera, camera->camera_buff, camera->max_image, &out_size);



   // Make sure word boundry is zero for checksum function

    camera->camera_buff[out_size + 0] = 0;
    camera->camera_buff[out_size + 1] = 0;
    camera->camera_buff[out_size + 2] = 0;
    camera->camera_buff[out_size + 3] = 0;



    short check = get_checksum16(camera->camera_buff, out_size);
   
         
      
//    alert(MODULE_NAME "Frame size: %d\n", out_size);

    transmit += out_size;
            

    // create the packet      
    char comm[] = { NC_IMAGE_SIZE,            // send compressed image
                    camera->image_x >> 8,    // x_size high byte
                    camera->image_x & 0xff,  // x_size low byte
                    camera->image_y >> 8,    // y_size high byte
                    camera->image_y & 0xff,  // y_size low byte
       
                    out_size >> 24,     // high     set the data _size
                    out_size >> 16,     //
                    out_size >> 8,      // 
                    out_size & 0xff,
                    
                    check >> 8,
                    check & 0xff
                                        
                    };   // low
 
 
    // Send information packet
    if (send(client_sd, comm, NC_IMAGE_SIZE_LEN, 0) == -1)
    {
        alert(MODULE_NAME "Send error\n");
        camera->camera_client_connected = 0;
        return(1);
    }
    
    // Send compressed frame
    if (send(client_sd, camera->camera_buff, out_size, 0) == -1)
    {
        alert(MODULE_NAME "Send error\n");
        camera->camera_client_connected = 0;
        return(1);
    }
    

    static int save_count = 0;

    if (save_image)
    {
        char fname[54];
        sprintf(fname, "image%010d.jpg", save_count++);


        FILE *f = fopen(fname, "wb");
        fwrite(camera->camera_buff, out_size, 1, f);
        fclose(f);
        
    }
        
   // log_write("Send complete\n");
    
    return(0);
}
/* End of send camera image */


/* Het data from the client */
int camera_get_data( camera_type *camera, int sd, char *data )
{
    return(recv(sd, data, REC_BUFFER, 0));
}
/* End of get_data */


/* Catch network error signal */
void catch_camera_neterror( int sig )
{
    alert(MODULE_NAME "Unable to send data\n");

//    camera->camera_client_connected = 0;

    signal (sig, catch_camera_neterror);
}
/* End of catch network error */


/* Init the network */
int setup_camera_server( camera_type *camera )
{
    signal (SIGPIPE, catch_camera_neterror);
    signal (SIGURG, catch_camera_neterror);

    // setup the camera first

    camera->camera_server_sd = socket(AF_INET, SOCK_STREAM, 0);

    if(camera->camera_server_sd < 0) 
    {
        alert(MODULE_NAME "Unable to open socket\n");
        return(1);
    }

    struct sockaddr_in server_addr; 

    memset(&server_addr, 0, sizeof(server_addr));

    server_addr.sin_family = AF_INET;
    server_addr.sin_addr.s_addr = INADDR_ANY;
    server_addr.sin_port = htons(camera->port);
  
      // set socket to resuse
    int reuse = 1;
    int reuse_len = sizeof(int);

    if (setsockopt(camera->camera_server_sd, SOL_SOCKET, SO_REUSEADDR, &reuse, reuse_len) == -1)
    {
        alert(MODULE_NAME "Unable to set REUSE\n");
        return(1);
    }
   
    if(bind(camera->camera_server_sd, (struct sockaddr *) &server_addr, sizeof(server_addr)) == -1) 
    {
        alert(MODULE_NAME "Unable to bind\n");
        return(1);
    }
    
    if(listen(camera->camera_server_sd, 1) == -1)  // listen for one only client
    {
        alert(MODULE_NAME "Unable to listen\n");
        return(1);
    }
 
    return(0);   
}
/* End of setup network */


/* Wait for a client to connect */
int camera_wait_for_client(camera_type *camera)
{
    //alert(MODULE_NAME "Waiting for connection on port %d\n", camera->port);

    struct sockaddr_in client_addr;
    socklen_t client_len = sizeof(struct sockaddr_in);

    return(accept(camera->camera_server_sd, (struct sockaddr *) &client_addr, &client_len));
}
/* End of wait for client */


/* Core loop for clients */
// BUG need to replace with stream processor
void camera_network_loop( camera_type *camera,int client_sd )
{
    camera->camera_client_connected = 1;    // we are now connected

    char data[REC_BUFFER];
        
    while(!program_done && camera->camera_client_connected) // n == 0, client disconnect
    {
        int n = recv(client_sd, data, REC_BUFFER, 0);

        if (n < 1) // , no data
        {
            alert(MODULE_NAME "Unable to get data\n");
            camera->camera_client_connected = 0;
        } else
        {
            // parse client commands
            if (data[0] == 0x40) // send image
            {
                //log_write("Send request\n");      
                if (send_camera_image(camera, client_sd))
                {
                    camera->camera_client_connected = 0;
                    alert(MODULE_NAME "send_camera_image fail\n");
                }
            }
        } 
        
      //  send_camera_image(client_sd);
        
        usleep(camera_delay);
    }

    close(client_sd);               

    camera->camera_client_connected = 0;    // no longer connected
}
/* End of camera_client_loop */


/* Core network thread */
void *camera_server( void *threadid )
{
    //alert(MODULE_NAME "Server started\n");

    camera_type *camera = (camera_type *) threadid;


    // core server loop
    while(!program_done) 
    {
        // client socket
        int client_sd = camera_wait_for_client(camera);

        if (client_sd < 0)
        {
            alert(MODULE_NAME "Unable to accept\n");
            break;
        }

        alert(MODULE_NAME "Client connected\n");

        // core loop
        // we will be creating a new process here for 
        // multiple clients
        camera_network_loop(camera, client_sd);

        alert(MODULE_NAME "client disconnect\n");
    }

    close(camera->camera_server_sd);
  
    alert(MODULE_NAME "Server shutdown\n");

    pthread_exit(NULL); // exit thread
}
/* End of camera_server  */


/* Start camera system */
void camera_start( camera_type *camera, char * device, int x_res, int y_res, int port )
{
    // camera_server_sd;  
    camera->camera_client_connected = 0;

    camera->image_x = 0; 
    camera->image_y = 0;

    camera->req_x = x_res; 
    camera->req_y = y_res;


    camera->max_image = 0;

    camera->save_image = 0;
    camera->save_count = 0;

    camera->port = port;
    strcpy(camera->dev_name, device);

    printf(MODULE_NAME "Loading camera %s port %d delay(%d)\n", camera->dev_name, camera->port, camera_delay);


    if (cameradrv_start (camera))
    {
        alert(MODULE_NAME "Unable to open camera %s\n", device);
        return;
    }

    camera->camera_buff = (unsigned char *) malloc(camera->max_image);


    if (setup_camera_server(camera))
    {
        alert(MODULE_NAME "Camera server setup failed\n");
        return;       
    }


    alert(MODULE_NAME "Loaded [%d,%d]\n", camera->image_x, camera->image_y);

    // take care of camera connections
    pthread_create(&camera->camera_t, NULL, camera_server, camera);
    
}
/* End of camera_start */
    

/* Stop camera system */
void camera_stop( camera_type *camera )
{
    close(camera->camera_server_sd);

    cameradrv_stop(camera);
    
    free(camera->camera_buff);
    
}
/* End of camera_stop */



/* low level camera */







