/*
	Serial Camera Client 
	Version 11/24/2010-A
	(C) 2010 Jason Hunt nulluser@gmail.com

	This program is free software; you can redistribute it and/or
    	modify it under the terms of the GNU General Public License
    	as published by the Free Software Foundation; either version 2
    	of the License, or (at your option) any later version.

    	This program is distributed in the hope that it will be useful,
    	but WITHOUT ANY WARRANTY; without even the implied warranty of
    	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    	GNU General Public License for more details.

    	You should have received a copy of the GNU General Public License
    	along with this program; if not, write to the Free Software
    	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

// Project Headers
#include "config.h"
#include "utility.h"
#include "camera.h"
#include "server.h"

// System Headers
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MODULE_NAME  "[Config] "
    
extern char camera_device[];
extern char serial_device[];

extern int  server_port;
extern int  camera_port;

extern int camera_delay;

extern int telemetry_size;



/* gets the nextr string in a line, up to a space or endline */
void get_next_string(char *line, unsigned int *i, char *str)
{
    int done_str = 0;   // Flag for done with string
    
    unsigned int s = 0; // Index into target string
   

    // Eat white space at front of line
    while ((line[*i] == ' ' || line[*i] == '\t') && *i < CONFIG_LINE_MAX) 
        (*i)++;
  
    while (!done_str && *i < CONFIG_LINE_MAX)
    {
        char ch = line[*i];

        (*i)++;
         
        if (ch == ' ' || ch == '\t' ||  ch == 0)
            done_str = 1;
        else
            str[s++] = ch;
    }

    str[s] = 0;   // null terminate
   
}
/* end of get_next_string */


/* parse a config line for commands */
void parse_line(char *line)
{
    unsigned int index = 0;
    char str[CONFIG_LINE_MAX];
    
    get_next_string(line, &index, str);
   
    if (!strcmp("serial_device", str))                   // set hostname
    {
        get_next_string(line, &index, str);
    
        if (strlen(str) > 0)
            strncpy(serial_device, str, MAX_STR);  
            
    }  else
    
    if (!strcmp("telemetry_size", str))                   // set hostname
    {
        get_next_string(line, &index, str);
        telemetry_size = atoi(str);  
            
    }  else

    if (!strcmp("camera_delay", str))                   // set hostname
    {
        get_next_string(line, &index, str);
        camera_delay = atoi(str);
    }

    if (!strcmp("camera", str))                   // set hostname
    {
        char name[1024];
        get_next_string(line, &index, name);
            
        get_next_string(line, &index, str);
        int x_res = atoi(str);

        get_next_string(line, &index, str);
        int y_res = atoi(str);
   
        get_next_string(line, &index, str);
        int port = atoi(str);
        
        
        if (num_cameras >= MAX_CAMERA)
        {
            alert(MODULE_NAME "MAX_CAMERA exceeded. Device: %s\n", name);
            return;
        }
           
        camera_start(&camera[num_cameras++], name, x_res, y_res, port);

    }     else
    
    if (!strcmp("server_port", str))                   // set hostname
    {
        get_next_string(line, &index, str);
    
        sscanf(str, "%d", &server_port);
    }       
    
 /*   if (!strcmp("camera_port", str))                   // set hostname
    {
        get_next_string(line, &index, str);
    
        sscanf(str, "%d", &camera_port);
    }       */
    
}
/* end of parse_line */


/* loads configuration values from config file */
void load_config( char *config_filename )
{
    FILE *f = fopen(config_filename, "rt");
    
    if (!f)
    {
        alert(MODULE_NAME "Unable to load config file\n");
        return;
    }    

    // load each line
    while (!feof(f))
    {
        char line[CONFIG_LINE_MAX];    // temperary line

        unsigned int i = 0;     // index
        
        int line_done = 0;      // line end
        
        // read line
        while (i < CONFIG_LINE_MAX && !line_done)
        {
            char ch = fgetc(f);             // get char from file

            // eat rest of line
            if (ch == '#')
            {
                while (ch != 0x0a && ch != 0x0d && !feof(f))  
                    ch = fgetc(f);
            } else    
    
            // limit check
            if (ch == 0x0d || ch == 0x0a || feof(f)) line_done = 1; else
                line[i++] = ch;
        }            
               
        line[i] = 0; // null terminate
        
        // check for blank line
        if (i > 0)
            parse_line(line);
           
           // parse_line(line);
           
    }    
    
    alert(MODULE_NAME "Loaded config %s\n", config_filename);
    
    
    fclose(f);
}
/* end of load config */



