








// Simple complementary filter
//    double angle_alpha = 0.98;
//    angle = (angle_alpha)*(angle + gyro.z*gyro_kz * dt) + (1- angle_alpha)*(angle_mag);



 /*
    const double	Q = 0.002;	// Noise weighting matrix 
    static double angle = 0;            // State
    static double R = .05;		// Measurement error weight 
    static double P = 1;		// Covariance matrix 
	double			Pdot;		// Derivative of P 
	double			E;		    //
	double			K;		    //
	double			theta_m;	// state measurement 
    						// A = 0 
	Pdot = Q;				// Pdot = A*P + P*A' + Q 
	P += Pdot * dt;
	angle += rate_gyro * dt;	// Update estimate from gyro
	E = P + R;				// E = CPC' + R 
	K = P / E;				// K = PC'inv(E) 
	angle += K * ( angle_mag - angle ); // Update state
	P *= ( 1 - K ); 	// Update Covariance   */
 
 


void updateFilter(double w_x, double w_y, double w_z, double a_x, double a_y, double a_z) 
{

    //Local system variables.

    //Vector norm.
    double norm;
    //Quaternion rate from gyroscope elements.
    double SEqDot_omega_1;
    double SEqDot_omega_2;
    double SEqDot_omega_3;
    double SEqDot_omega_4;
    //Objective function elements.
    double f_1;
    double f_2;
    double f_3;
    //Objective function Jacobian elements.
    double J_11or24;
    double J_12or23;
    double J_13or22;
    double J_14or21;
    double J_32;
    double J_33;
    //Objective function gradient elements.
    double nablaf_1;
    double nablaf_2;
    double nablaf_3;
    double nablaf_4;



    //Quaternion orientation of earth frame relative to auxiliary frame.
     static double AEq_1 = 1, AEq_2 = 0, AEq_3 = 0, AEq_4 = 0;
    
    //Estimated orientation quaternion elements with initial conditions.
    static double SEq_1 = 1, SEq_2 = 0, SEq_3 = 0, SEq_4 = 0;



    static double beta = 0.001;//sqrt(3.0 / 4.0) * (M_PI * (0.01 / 180.0));


    double deltat = 0.01;
    
    static int firstUpdate = 0;





    //Auxiliary variables to avoid reapeated calcualtions.
    double halfSEq_1 = 0.5 * SEq_1;
    double halfSEq_2 = 0.5 * SEq_2;
    double halfSEq_3 = 0.5 * SEq_3;
    double halfSEq_4 = 0.5 * SEq_4;
    double twoSEq_1 = 2.0 * SEq_1;
    double twoSEq_2 = 2.0 * SEq_2;
    double twoSEq_3 = 2.0 * SEq_3;

    //Compute the quaternion rate measured by gyroscopes.
    SEqDot_omega_1 = -halfSEq_2 * w_x - halfSEq_3 * w_y - halfSEq_4 * w_z;
    SEqDot_omega_2 = halfSEq_1 * w_x + halfSEq_3 * w_z - halfSEq_4 * w_y;
    SEqDot_omega_3 = halfSEq_1 * w_y - halfSEq_2 * w_z + halfSEq_4 * w_x;
    SEqDot_omega_4 = halfSEq_1 * w_z + halfSEq_2 * w_y - halfSEq_3 * w_x;

    //Normalise the accelerometer measurement.
    norm = sqrt(a_x * a_x + a_y * a_y + a_z * a_z);
    a_x /= norm;
    a_y /= norm;
    a_z /= norm;

    //Compute the objective function and Jacobian.
    f_1 = twoSEq_2 * SEq_4 - twoSEq_1 * SEq_3 - a_x;
    f_2 = twoSEq_1 * SEq_2 + twoSEq_3 * SEq_4 - a_y;
    f_3 = 1.0 - twoSEq_2 * SEq_2 - twoSEq_3 * SEq_3 - a_z;
    //J_11 negated in matrix multiplication.
    J_11or24 = twoSEq_3;
    J_12or23 = 2 * SEq_4;
    //J_12 negated in matrix multiplication
    J_13or22 = twoSEq_1;
    J_14or21 = twoSEq_2;
    //Negated in matrix multiplication.
    J_32 = 2 * J_14or21;
    //Negated in matrix multiplication.
    J_33 = 2 * J_11or24;

    //Compute the gradient (matrix multiplication).
    nablaf_1 = J_14or21 * f_2 - J_11or24 * f_1;
    nablaf_2 = J_12or23 * f_1 + J_13or22 * f_2 - J_32 * f_3;
    nablaf_3 = J_12or23 * f_2 - J_33 * f_3 - J_13or22 * f_1;
    nablaf_4 = J_14or21 * f_1 + J_11or24 * f_2;

    //Normalise the gradient.
    norm = sqrt(nablaf_1 * nablaf_1 + nablaf_2 * nablaf_2 + nablaf_3 * nablaf_3 + nablaf_4 * nablaf_4);
    nablaf_1 /= norm;
    nablaf_2 /= norm;
    nablaf_3 /= norm;
    nablaf_4 /= norm;

    //Compute then integrate the estimated quaternion rate.
    SEq_1 += (SEqDot_omega_1 - (beta * nablaf_1)) * deltat;
    SEq_2 += (SEqDot_omega_2 - (beta * nablaf_2)) * deltat;
    SEq_3 += (SEqDot_omega_3 - (beta * nablaf_3)) * deltat;
    SEq_4 += (SEqDot_omega_4 - (beta * nablaf_4)) * deltat;

    //Normalise quaternion
    norm = sqrt(SEq_1 * SEq_1 + SEq_2 * SEq_2 + SEq_3 * SEq_3 + SEq_4 * SEq_4);
    SEq_1 /= norm;
    SEq_2 /= norm;
    SEq_3 /= norm;
    SEq_4 /= norm;

    if (firstUpdate == 0) {
        //Store orientation of auxiliary frame.
        AEq_1 = SEq_1;
        AEq_2 = SEq_2;
        AEq_3 = SEq_3;
        AEq_4 = SEq_4;
        firstUpdate = 1;
    }


   //Quaternion describing orientation of sensor relative to earth.
    double ESq_1, ESq_2, ESq_3, ESq_4;
    //Quaternion describing orientation of sensor relative to auxiliary frame.
    double ASq_1, ASq_2, ASq_3, ASq_4;    
                              
    //Compute the quaternion conjugate.
    ESq_1 = SEq_1;
    ESq_2 = -SEq_2;
    ESq_3 = -SEq_3;
    ESq_4 = -SEq_4;

    //Compute the quaternion product.
    ASq_1 = ESq_1 * AEq_1 - ESq_2 * AEq_2 - ESq_3 * AEq_3 - ESq_4 * AEq_4;
    ASq_2 = ESq_1 * AEq_2 + ESq_2 * AEq_1 + ESq_3 * AEq_4 - ESq_4 * AEq_3;
    ASq_3 = ESq_1 * AEq_3 - ESq_2 * AEq_4 + ESq_3 * AEq_1 + ESq_4 * AEq_2;
    ASq_4 = ESq_1 * AEq_4 + ESq_2 * AEq_3 - ESq_3 * AEq_2 + ESq_4 * AEq_1;

    //Compute the Euler angles from the quaternion.
 double    phi = 180/M_PI * atan2(2 * ASq_3 * ASq_4 - 2 * ASq_1 * ASq_2, 2 * ASq_1 * ASq_1 + 2 * ASq_4 * ASq_4 - 1);
double     theta = 180/M_PI * asin(2 * ASq_2 * ASq_3 - 2 * ASq_1 * ASq_3);
double     psi = 180/M_PI * atan2(2 * ASq_2 * ASq_3 - 2 * ASq_1 * ASq_4, 2 * ASq_1 * ASq_1 + 2 * ASq_2 * ASq_2 - 1);



printf("Phi:  %8.4f Theta:  %8.4f  Psi:  %8.4f\n", phi, theta, psi);

}

 
 

    const float Sz = 0.055; // Variance of mag data

    // Data read from 1000 samples of the gyroscope had a variance of 0.00025556.
    // XXX The values below were pulled from the autopilot site, but I'm not sure how to
    // XXX plug them into the process noise covariance matrix.  This needs to be
    // XXX further explored.


    const float Sw_00 = 0.0374;
    const float Sw_01 = 0.0374;
    const float Sw_10 = 0.0374;
    const float Sw_11 = 0.0374;

 
 
 
 
 
void  kalman_update(float dt, float gyro_rate, float accel_angle, float *angle)
{
    // Inputs.
    float u = gyro_rate;
    float y = accel_angle;

    // Output.

    float     x_00 = *angle;
    static float x_10 = 0.0;


    // Persistant states.
    static float P_00 = 0.000;
    static float P_01 = 0.000;
    static float P_10 = 0.000;
    static float P_11 = 0.000;


    // dT between samples
    float A_01 = -dt;
    float B_00 = dt;


    // Update the state estimate by extrapolating current state estimate with input u.
    // x = A * x + B * u
    x_00 += (A_01 * x_10) + (B_00 * u);

    // Compute the innovation -- error between measured value and state.
    // inn = y - c * x
    float inn_00 = y - x_00;

    // Compute the covariance of the innovation.
    // s = C * P * C' + Sz
    float s_00 = P_00 + Sz;

    // Compute AP matrix for use below.
    // AP = A * P
    float AP_00 = P_00 + A_01 * P_10;
    float AP_01 = P_01 + A_01 * P_11;
    float AP_10 = P_10;
    float AP_11 = P_11;

    // Compute the kalman gain matrix.
    // K = A * P * C' * inv(s)
    float K_00 = AP_00 / s_00;
    float K_10 = AP_10 / s_00;
    
    // Update the state estimate.
    // x = x + K * inn
    x_00 += K_00 * inn_00;
    x_10 += K_10 * inn_00;

    // Compute the new covariance of the estimation error.
    // P = A * P * A' - K * C * P * A' + Sw
    float APAT_00 = AP_00 + (AP_01 * A_01);
    float     APAT_01 = AP_01;
    float     APAT_10 = AP_10 + (AP_11 * A_01);
    float     APAT_11 = AP_11;
    
    
    
    float     KCPAT_00 = (K_00 * P_00) + (K_00 * P_01) * A_01;
    float     KCPAT_01 = (K_00 * P_01);
    float     KCPAT_10 = (K_10 * P_00) + (K_10 * P_01) * A_01;
    float     KCPAT_11 = (K_10 * P_01);
    
    
    
    
    P_00 = APAT_00 - KCPAT_00 + Sw_00;
    P_01 = APAT_01 - KCPAT_01 + Sw_01;
    P_10 = APAT_10 - KCPAT_10 + Sw_10;
    P_11 = APAT_11 - KCPAT_11 + Sw_11;

    // Return the estimate.
//    return x_00;

*angle = x_00;
}

 
 
 
 
 
 
 
 void robot_imu_kalman( unsigned char *buffer)
 {
    //show_packet("Tele Buffer: ", "", buffer, COMM_IN_TEL_LEN);

    float alpha = 0.3;
    
    
    static int zero_init = 0;
    static int init = 0;
    

    static double last_time;
 //   static double start_time;
    
    double dt;
    double current;
   
    
    static vector accl = {0,0,0};
    static vector mag  = {0,0,0};
    static vector gyro = {0,0,0};
    
    static vector accl_zero = {0,0,0};
    static vector mag_zero = {0,0,0};
    static vector gyro_zero = {0,0,0};


    current = get_time();
   
    dt = current - last_time;
    last_time = current;

    
    vector accl_act, mag_act, gyro_act;          
       
    get_acc(buffer,       &accl_act);
    get_mag(buffer + 6,   &mag_act);
    get_gyro(buffer + 12, &gyro_act);

	static float angle = 0;
	
    double gyro_kz = 500;


    // Measurements
    double mag_angle = atan2(mag_act.z, mag_act.x) * 180.0 / M_PI;
    
    
    double gyro_rate = -gyro_act.z * gyro_kz;

	

    // First run, init
    if (init == 0)
    {
        init = 1;
        last_time = get_time();
        
        angle = mag_angle;
        
        //start_time = last_time;
        return;
    }


/*    // Get sensor zeros after a few seconds
    if (zero_init == 0 && current > start_time + 2.0)
    {
        printf("zero\n");
    
        zero_init = 1;
        
        accl_zero = accl;
        mag_zero = mag;
        gyro_zero = gyro;    
    }*/
    
/*

    accl_act = vector_sub(accl_act, accl_zero);
    mag_act = vector_sub(mag_act, mag_zero);
    gyro_act = vector_sub(gyro_act, gyro_zero);

    filter(&accl, accl_act, alpha);
    filter(&mag,  mag_act,  alpha);
    filter(&gyro, gyro_act, alpha);*/




    double a = 10;
    double noise =  2*a *  (rand() % 1000) / 1000.0 - a;
	//mag_angle += noise;
	

	
	
    kalman_update (dt, gyro_rate, mag_angle, &angle);
	
	
	
	
	
	
	double diff = mag_angle - angle;
	

/*

    static double var_mag_angle = 0;
    static double ave_mag_angle = 0;

    static  double t = 1;
    
    
    ave_mag_angle = ((t-1)/t) * ave_mag_angle + (1/t) * mag_angle;


    if (t < 2) 
        var_mag_angle = 0;
    else
        var_mag_angle = ((t-1)/t) * var_mag_angle  + (1/(t-1)) * pow(mag_angle - ave_mag_angle,2);
 
    t += 1.0;

    printf("mag_ang: %+08.4f   Mave: %6.3f  Mvar: %6.6f\n",  mag_angle, ave_mag_angle, var_mag_angle);

*/
/*
    static double var_gyro_rate = 0;
    static double ave_gyro_rate = 0;

    static  double t = 1;
    
    
    ave_gyro_rate = ((t-1)/t) * ave_gyro_rate + (1/t) * gyro_rate;


    if (t < 2) 
        var_gyro_rate = 0;
    else
        var_gyro_rate = ((t-1)/t) * var_gyro_rate  + (1/(t-1)) * pow(gyro_rate - ave_gyro_rate,2);
 
    t += 1.0;

    printf("gyro_rate %+08.4f   Gave: %6.3f  Gvar: %6.6f",  gyro_rate, ave_gyro_rate, var_gyro_rate);

*/






    printf("mag_ang: %+08.4f  gyro_rate: %+08.4f  e_ang: %+08.4f diff: %+08.4f ", 
            mag_angle,  gyro_rate , angle, diff );


//printf("P00 %8.4f P01 %8.4f P10 %8.4f P11 %8.4f ", P_00, P_01, P_10, P_11);



//printf("k0 %8.4f k1%8.4f  s: %8.4f", K_0, K_1, S);


//    printf("A  x: %4d y: %4d z: %4d   ",a_x.n,a_y.n,a_z.n);
//    printf("A  x: %6.3f y: %6.3f z: %6.3f  ", accl.x, accl.y, accl.z);

//    printf("M  x: %4d y: %4d z: %4d  ",m_x.n,m_y.n,m_z.n);
//    printf("M  x: %6.3f y: %6.3f z: %6.3f  ",mag.x, mag.y, mag.z);

//    printf("G  x: %4d y: %4d z: %4d\n",g_x.n, g_y.n, g_z.n);
//    printf("G  x: %6.3f y: %6.3f z: %6.3f  ", gyro.x, gyro.y, gyro.z);

             /*       
    float pitch, roll, yaw;

//    float vlen = sqrt(ax*ax + ay*ay + az*az);

    roll = atan2(accl.z, accl.y) * 180 / M_PI - 90.0;
    pitch = atan2(accl.z, accl.x) * 180 / M_PI - 90.0;

    yaw = 0;
*/

   // printf("A  x: %6.3f y: %6.3f z: %6.3f  Len:  %6.3f   Roll: %6.2f  Pitch:  %6.2f  Yaw:  %6.2f\n ",ax,ay,az,vlen, roll, pitch, yaw);

    printf("\n");

 }
 


