/*
    NullBot Robot Platform - Server - 2005-2011 Jason Hunt
    
    GNU GENERAL PUBLIC LICENSE Version 2, June 1991
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: nulluser@gmail.com
    
    File: network.c
*/

#include <stdio.h>
#include <sys/types.h> 
#include <sys/socket.h>
#include <netinet/in.h>

#include <pthread.h>
#include <fcntl.h> 
#include <unistd.h>
#include <stdio.h>
#include <memory.h>
#include <errno.h>
#include <signal.h>
#include <stdlib.h>

#include "../version.h"
#include "../netcomm.h"

#include "utility.h"
#include "server.h"
#include "camera.h"
#include "failsafe.h"
#include "robot.h"
#include "network.h"

#define REC_BUFFER  1024

#define MODULE_NAME "[Network] "


int server_sd;              // server socket
int connected = 0;          // connected flag

pthread_t network_t;
  

unsigned int updates = 0;
 
int current_client  = 0;

int server_port = 0;

 
/* Return true if a client is connected */
unsigned int network_connected( void )
{
    return connected;
}
/* End of network_connected */








/* Preform a transfer with the controller */
int network_write_packet( unsigned char packet_type, 
                          unsigned char *data, 
                          int len )
{

    if (!connected) return 1;
    
    unsigned char buff[2] = {len + 3, packet_type};        // Telemetry data
    
    // Send packet header
    if (send(current_client, buff, 2,0 ) == -1) return(1);

    // Send packet data
    if (send(current_client, data, len,0 ) == -1) return(1);
    
    
    // Set checksum
    buff[0] =  get_checksum(data, len) + 
               get_checksum(buff, 2) ;

    // Send packet footer
//    if (serial_send (data, 1) == -1) return(1);
    if (send(current_client, buff, 1,0 ) == -1) return(1);
                  
    return(0);
}
/* End of network_write_packet */





/* Send the controller data back to the client */
void return_controller_data( int sd )
{
/*    char data[5];
    data[0] = 0x30;     // send string to client
    
    memcpy(data+1, c_data, 4); // copy the controller data

    if (send(sd, data, 5, 0) == -1)
    {
        connected = 0;
        alert(MODULE_NAME "Unable to send data\n");
    }*/
            
}
/* End of send_controller_data */
                      
         
/* Get data from the client */
int get_data( int sd, char *data )
{
    return(recv(sd, data, REC_BUFFER, 0));
}
/* End of get_data */





/* Send version information to client */
/*void send_packet(unsigned char *data, unsigned char len )
{
    if (!connected) return;

    unsigned char total_len = len + 3;
    unsigned char *buffer = (unsigned char *) malloc(total_len);
    
    buffer[0] = total_len;
    buffer[1] = 0x40;
        
    memcpy(buffer + 2, data, len);
                                                       
                    
    buffer[total_len-1] = get_checksum(buffer, total_len-1);
        
        
        show_packet("To CLient", buffer, total_len);
        
    if (send(current_client, buffer, total_len,0 ) == -1)
    {
        connected = 0;
        alert(MODULE_NAME "(send_packet) Unable to send telemetry\n");
    }
    
       
    free(buffer);
        
}*/
/* End of send string */









/* Send version information to client */
void send_string( int sd, char *str )
{
  /*  int len = strlen(str);
    if (len > 255) len = 255;

    char data[len+2];

    data[0] = 0x20;     // send string to client
    data[1] = len;

    memcpy(data+2, str, len); // copy the string to the packet buffer
   
    if (send(sd, data, len + 2, 0) == -1)
    {
        connected = 0;
        alert(MODULE_NAME "Unable to send string\n");
    } */    
}
/* End of send string */


/* Send version information to client */
void send_version( int sd )
{
 //   send_string(sd, VERSION);
}
/* End of send version */


/* SIGPIPE handle */
void catch_control_neterror( int sig )
{
    alert(MODULE_NAME "Unable to send data\n");

    connected = 0; 
    
    signal (sig, catch_control_neterror);
}
/* End of  catch_control_neterror */


/* Init the network */
int setup_network( void )
{
    // first catch error signals
    signal (SIGPIPE, catch_control_neterror);

    server_sd = socket(AF_INET, SOCK_STREAM, 0);

    if(server_sd < 0) 
    {
        alert(MODULE_NAME "Unable to open socket\n");
        return(1);
    }
 
    struct sockaddr_in server_addr;  // server IP structure
    memset(&server_addr, 0, sizeof(server_addr));
    server_addr.sin_family = AF_INET;
    server_addr.sin_addr.s_addr = INADDR_ANY;
    server_addr.sin_port = htons(server_port);
 

    // set socket to resuse
    int reuse = 1;
    int reuse_len = sizeof(int);
    if (setsockopt(server_sd, SOL_SOCKET, SO_REUSEADDR, &reuse, reuse_len) == -1)
    {
        alert(MODULE_NAME "Unable to ser REUSE\n");
        return(1);
    }

    if(bind(server_sd, (struct sockaddr *) &server_addr, sizeof(server_addr)) == -1) 
    {
        alert(MODULE_NAME "Unable to bind\n");
        alert(MODULE_NAME "Error: %d\n",errno);

        return(1);
    }
            
    // listen for one client
    if(listen(server_sd, 1) == -1)  // listen for one only client
    {
        alert(MODULE_NAME "Unable to listen\n");
        return(1);
    }
 
    return(0);   
}
/* End of setup network */


/* Wait for a client to connect */
int wait_for_client( void )
{
    alert(MODULE_NAME "Waiting for connection on port %d\n", server_port);

    struct sockaddr_in client_addr;
    socklen_t client_len = sizeof(client_addr);

    return(accept(server_sd, (struct sockaddr *) &client_addr, &client_len));
}
/* End of wait for client */


/* Shutdown computer */
void shut_down( void )
{
    system("poweroff");   
}
/* End of shutdown */


/* The main network loop */

//  BUG this has to be converted to a byte stream processor


void control_network_loop( int client_sd )
{
    connected = 1;  // we are now connected
            
    // send our version data to the client
    send_version(client_sd);


    current_client = client_sd;


    // data buffer for control commands
    char data[REC_BUFFER];
        
    while(!program_done && connected) // n == 0, client disconnect
    {
        int n = get_data(client_sd, data);
        if (n < 1)
        {
            //alert(MODULE_NAME "Unable to get data\n");
            //connected = 0;       
        } else
        {   // we have data



        //            printf("D: %x\n", data[0]);
            //parse client commands
            // test first command byte
            if (data[0] == CONTROL_KEEP_ALIVE) // keep alive from client
            {
              //update_controller(); 
               // update_count++;
            } 
            if (data[0] == CONTROL_SHUTDOWN) // keep alive from client
            {
                send_string(client_sd, "Server is shutting down now");
                
                robot_reset();
                shut_down();
            } 
            if (data[0] == CONTROL_FIREON) // keep alive from client
            {
//                fire = 1;
               // alert(MODULE_NAME "Fire on\n");
            } 
            if (data[0] == CONTROL_FIREOFF) // keep alive from client
            {
               // fire = 0;
              //  alert("Fire off\n");
            } 
            if (data[0] == CONTROL_VERSION) // send version
            {
                send_version(client_sd);
            } 
            if (data[0] == CONTROL_STRING) // display string
            {
                unsigned char len = data[1];
                //alert("Len: %d\n", len);
                memcpy(data, data+2, len); // copy over command byte
                data[len] = 0;
                alert(MODULE_NAME "Message from client: %s\n", data); // display the string
            } else 
            if (data[0] == CONTROL_UPDATE) // control update
            {
                // make sure we have all of the data first
                if (n < 5)
                {
                    alert(MODULE_NAME "Invalid control packet: %d\n", n);
                    connected = 0;
                    return;
                }
               
                 
                robot_set_control((unsigned char *)(data+1), 4);
                

           /*     alert(MODULE_NAME "Data: %X %X %X %X\n", 
                                    (unsigned char)data[1], (unsigned char)data[2], 
                                    (unsigned char)data[3], (unsigned char)data[4]);                    
*/
                   

                //update_controller();    // send the net vaules to the motor controller

               // return_controller_data(client_sd); //send the values back to the client
                   

            }
        } 
    }

    robot_reset();

    // we are disconneced
    close(client_sd);
    connected = 0;    
}
/* End of control_network_loop */


/* Core network thread */
void *control_network( void *threadid )
{
    alert(MODULE_NAME "Started\n");


    while(!program_done) 
    {
        // reset controller on new connection, might happen twice
        robot_reset();

        // wait for the client to connect
        int client_sd =  wait_for_client();

        if (client_sd < 0) 
        {
            alert(MODULE_NAME "Unable to accept\n");
            break;
        }

        alert(MODULE_NAME "Client connected\n");

        control_network_loop(client_sd);
    }

    // program has exited
    close(server_sd);
   
    alert(MODULE_NAME "Shutdown\n");

    pthread_exit(NULL); // exit thread
}
/* End of control_network */


/* Start telemetry and control server */
void network_start( void )
{
    if (setup_network())
    {
        alert(MODULE_NAME "Network setup failed\n");
        return;
    }

    pthread_create(&network_t, NULL, control_network, NULL);
}
/* End of network_start */


/* Stop telemetry and control server */
void network_stop( void )
{
    close(server_sd);
}
/* End of network_stop */




