/*
    NullBot Robot Platform - Server - 2005-2011 Jason Hunt
    
    GNU GENERAL PUBLIC LICENSE Version 2, June 1991
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: nulluser@gmail.com
    
    File: robot.c
*/

#include <stdio.h>
#include <memory.h>
#include <termios.h> 
#include <fcntl.h> 
#include <unistd.h>
#include <pthread.h>
#include <math.h>
#include <stdlib.h>


#include "../netcomm.h"
#include "utility.h"
#include "serial.h"
#include "failsafe.h"
#include "server.h"
#include "robot.h"
#include "network.h"

#define MODULE_NAME "[Robot] "

#define ACC_MAXVAL   (4095.0)
#define MAG_MAXVAL   (4095.0)
#define GYRO_MAXVAL   (65535.0)


#define MDRIVER     1





robot_type robot;


    double phi = 0;
    double theta = 0;
	double psi = 0;

int telemetry_size = 0;


void robot_get_wifi_strength ( void )
{
    FILE *f = popen(" awk -F'[ :]*' '/:/ {gsub(/\\. /,\" \",$0); print $4; }' /proc/net/wireless","r");

    if(f == NULL)
    {
        alert("Unable to open wifi file\n");
        return;
    }

    unsigned int quality;

    fscanf(f, "%d", &quality);

 //   alert("Link: %3.0f%%\n",  quality/LINK_QUALITY_MAX*100 );

    robot.wifi_link = (unsigned int)(quality * 255 /LINK_QUALITY_MAX);

    fclose(f);   
}



/* Return true if the client has been updated */
unsigned int robot_check_client( void )
{
    unsigned int tmp = robot.client_updates;
    
    robot.client_updates  = 0;
    
    return tmp;
}
/* End of Robot_check_client */


/* Return true if the client has been updated */
unsigned int robot_check_serial( void )
{
    unsigned int tmp = robot.serial_updates;
    
    robot.serial_updates  = 0;
    
    return tmp;
}


typedef union 
{
    float f;
    struct
    {
        unsigned char b0 : 8;
        unsigned char b1 : 8;
        unsigned char b2 : 8;
        unsigned char b3 : 8;
        
    };
} float_32;





/* End of robot_check_serial */
void robot_send_telemetry( void )
{
//    alert(MODULE_NAME "send_telemetry\n");
  

    unsigned char data_size = 0;
    unsigned char buffer[255];


    memcpy(buffer, robot.raw_telemetry, telemetry_size);


    data_size += telemetry_size;

    robot_get_wifi_strength();

    
//printf("id: %d\n", data_size);

    
    buffer[data_size++] = robot.wifi_link;


    float_32 t;
    
     t.f = phi;

    buffer[ data_size++] = t.b0;
    buffer[ data_size++] = t.b1;
    buffer[ data_size++] = t.b2;
    buffer[ data_size++] = t.b3;
        
    t.f = theta;

    buffer[ data_size++] = t.b0;
    buffer[ data_size++] = t.b1;
    buffer[ data_size++] = t.b2;
    buffer[ data_size++] = t.b3;
         
    t.f = psi;

    buffer[ data_size++] = t.b0;
    buffer[ data_size++] = t.b1;
    buffer[ data_size++] = t.b2;
    buffer[ data_size++] = t.b3;



    network_write_packet(COMM_NC_TEL, buffer, data_size);
}




typedef union 
{
    short n;
    struct
    {
        unsigned char l : 8;
        unsigned char h : 8;
    };
} int_16;







void get_acc( unsigned char *buffer, vector *act)
{
    int_16 a_x, a_y, a_z;
    
    a_x.l = buffer[0];    a_x.h = buffer[1];
    a_y.l = buffer[2];    a_y.h = buffer[3];
    a_z.l = buffer[4];    a_z.h = buffer[5];

    act->x = (float)(a_x.n >> 4) / ACC_MAXVAL;
    act->y = (float)(a_y.n >> 4) / ACC_MAXVAL;
    act->z = (float)(a_z.n >> 4) / ACC_MAXVAL;
}



void get_mag( unsigned char *buffer, vector *act)
{
    int_16 vx, vy, vz;
    
    vx.h = buffer[0];    vx.l = buffer[1];
    vz.h = buffer[2];    vz.l = buffer[3];  // z and y reversed
    vy.h = buffer[4];    vy.l = buffer[5];

    act->x = (float)(vx.n) / MAG_MAXVAL;
    act->y = (float)(vy.n) / MAG_MAXVAL;
    act->z = (float)(vz.n) / MAG_MAXVAL;
}




void get_gyro (unsigned char *buffer, vector *act)
{
    int_16 vx, vy, vz;
    
    vx.l = buffer[0];    vx.h = buffer[1];
    vy.l = buffer[2];    vy.h = buffer[3];
    vz.l = buffer[4];    vz.h = buffer[5];

    act->x = (float)(vx.n) / GYRO_MAXVAL;
    act->y = (float)(vy.n) / GYRO_MAXVAL;
    act->z = (float)(vz.n) / GYRO_MAXVAL;
}




void  filter(vector *current, vector actual, float alpha)
{
    current->x = alpha * actual.x + (1 - alpha) * current->x;
    current->y = alpha * actual.y + (1 - alpha) * current->y;
    current->z = alpha * actual.z + (1 - alpha) * current->z;
    
}



 
 





typedef  struct
{
    double v_00, v_01, v_02;
    double v_10, v_11, v_12;
    double v_20, v_21, v_22;
} matrix;






double vect_length(vector v)
{
    return sqrt(v.x * v.x + v.y*v.y + v.z*v.z);
}



vector vect_norm(vector v)
{
    vector r = v;

    double l = vect_length(v);

    if (l > 0)
    {
        r.x /= l;
        r.y /= l;
        r.z /= l;
    }
    
    return r;
    
}



vector mult_matrix_vector(matrix m, vector v)
{
    vector r;

    r.x = m.v_00 * v.x +  m.v_01 * v.y +  m.v_02 * v.z;
    r.y = m.v_10 * v.x +  m.v_11 * v.y +  m.v_12 * v.z;
    r.z = m.v_20 * v.x +  m.v_21 * v.y +  m.v_22 * v.z;

    return r;
}

 
 
 
 vector vector_sub(vector l, vector r)
 {
     vector v;
     
     v.x = l.x - r.x;
     v.y = l.y - r.y;
     v.z = l.z - r.z;
     
     return v;
 }
 
 
 

vector vector_cross(vector l, vector r)
 {
     vector v;
     
     v.x = l.y* r.z - l.z * r.y;
     v.y = l.z* r.x - l.x * r.z;
     v.z = l.x* r.y - l.y * r.x;
     
     return v;
 }
 
 


double vector_dot(vector l, vector r)
{
    double d;
    
    d = l.x * r.x + l.y * r.y + l.z * r.z;

    return d;
}
 
 
#define Kp 10.0f		// proportional gain governs rate of convergence to accelerometer/magnetometer
#define Ki 0.4f		// integral gain governs rate of convergence of gyroscope biases

//---------------------------------------------------------------------------------------------------
// Variable definitions

float q0 = 1, q1 = 0, q2 = 0, q3 = 0;	// quaternion elements representing the estimated orientation
float exInt = 0, eyInt = 0, ezInt = 0;	// scaled integral error






void AHRSupdate(float dt, vector g, vector a, vector m) 
{
	float norm;
	float hx, hy, hz, bx, bz;
	float vx, vy, vz, wx, wy, wz;
	float ex, ey, ez;

    float halfT = dt / 2.0;

	// auxiliary variables to reduce number of repeated operations
	float q0q0 = q0*q0, q0q1 = q0*q1, q0q2 = q0*q2, q0q3 = q0*q3;
	float q1q1 = q1*q1, q1q2 = q1*q2, q1q3 = q1*q3;
	float q2q2 = q2*q2, q2q3 = q2*q3, q3q3 = q3*q3;          
	
	// normalise the measurements
	norm = sqrt(a.x*a.x + a.y*a.y + a.z*a.z);       
	a.x = a.x / norm;
	a.y = a.y / norm;
	a.z = a.z / norm;

	norm = sqrt(m.x*m.x + m.y*m.y + m.z*m.z);          
	m.x = m.x / norm;
	m.y = m.y / norm;
	m.z = m.z / norm;         
	
	// compute reference direction of flux
	hx = 2*m.x*(0.5 - q2q2 - q3q3) + 2*m.y*(q1q2 - q0q3)        + 2*m.z*(q1q3 + q0q2);
	hy = 2*m.x*(q1q2 + q0q3)       + 2*m.y*(0.5 - q1q1 - q3q3)  + 2*m.z*(q2q3 - q0q1);
	hz = 2*m.x*(q1q3 - q0q2)       + 2*m.y*(q2q3 + q0q1)        + 2*m.z*(0.5 - q1q1 - q2q2);         

	bx = sqrt((hx*hx) + (hy*hy));
	bz = hz;        
	
	// estimated direction of gravity and flux (v and w)
	vx = 2*(q1q3 - q0q2);
	vy = 2*(q0q1 + q2q3);
	vz = q0q0 - q1q1 - q2q2 + q3q3;

	wx = 2*bx*(0.5 - q2q2 - q3q3) + 2*bz*(q1q3 - q0q2);
	wy = 2*bx*(q1q2 - q0q3) + 2*bz*(q0q1 + q2q3);
	wz = 2*bx*(q0q2 + q1q3) + 2*bz*(0.5 - q1q1 - q2q2);  
	
	// error is sum of cross product between reference direction of fields and direction measured by sensors
	ex = (a.y*vz - a.z*vy) + (m.y*wz - m.z*wy);
	ey = (a.z*vx - a.x*vz) + (m.z*wx - m.x*wz);
	ez = (a.x*vy - a.y*vx) + (m.x*wy - m.y*wx);
	
	// integral error scaled integral gain
	exInt = exInt + ex*Ki;
	eyInt = eyInt + ey*Ki;
	ezInt = ezInt + ez*Ki;
	
	// adjusted gyroscope measurements
	g.x = g.x + Kp*ex + exInt;
	g.y = g.y + Kp*ey + eyInt;
	g.z = g.z + Kp*ez + ezInt;
	
	// integrate quaternion rate and normalise
	q0 = q0 + (-q1*g.x - q2*g.y - q3*g.z)*halfT;
	q1 = q1 + ( q0*g.x + q2*g.z - q3*g.y) *halfT;
	q2 = q2 + ( q0*g.y - q1*g.z + q3*g.x) *halfT;
	q3 = q3 + ( q0*g.z + q1*g.y - q2*g.x) *halfT;  
	
	// normalise quaternion
	norm = sqrt(q0*q0 + q1*q1 + q2*q2 + q3*q3);

	q0 = q0 / norm;
	q1 = q1 / norm;
	q2 = q2 / norm;
	q3 = q3 / norm;
	
	
	
//	printf("q [%f, %f, %f, %f]", q0, q1, q2, q3);

	
   phi = 180 / M_PI * atan2(2*(q0*q1 + q2*q3), 1-2*(q1*q1 + q2*q2));
    theta = 180 / M_PI * asin(2*(q0*q2 - q3*q1));
psi = 180 / M_PI * atan2(2*(q0*q3 + q1*q2), 1-2*(q2*q2 + q3*q3));



    if (phi > 0) phi -= 180; else
phi += 180;
    



}







void robot_imu( unsigned char *buffer)
 {
    //show_packet("Tele Buffer: ", "", buffer, COMM_IN_TEL_LEN);

    float alpha = 0.3;
    
    
//    static int zero_init = 0;
//    static int init = 0;
    

    static double last_time;
//    static double start_time;
    
    double dt;
    double current;
   
    
    static vector accl = {0,0,0};
    static vector mag  = {0,0,0};
    static vector gyro = {0,0,0};
    
//    static vector accl_zero = {0,0,0};
//    static vector mag_zero = {0,0,0};
//    static vector gyro_zero = {0,0,0};


    current = get_time();
   
    dt = current - last_time;
    last_time = current;

    
    vector accl_act, mag_act, gyro_act;          
       
    get_acc(buffer,       &accl_act);
    get_mag(buffer + 6,   &mag_act);
    get_gyro(buffer + 12, &gyro_act);
    

    accl_act = vect_norm(accl_act);
    mag_act = vect_norm(mag_act);

    filter(&accl, accl_act, alpha);
    filter(&mag,  mag_act,  alpha);
    filter(&gyro, gyro_act, alpha);
   
    
        
    
    gyro_act.x *= -1;
    gyro_act.y *= -1;
    gyro_act.z *= -1;
    
    
//    updateFilter(gyro_act.x, gyro_act.y, gyro_act.z, mag_act.x, mag_act.y, mag_act.z);
   
    
    
//    printf("A [%6.3f,%6.3f,%6.3f] ", accl.x, accl.y, accl.z);
//    printf("M [%6.3f,%6.3f,%6.3f] ", mag.x, mag.y, mag.z);
//    printf("G [%6.3f,%6.3f,%6.3f] ", gyro.x, gyro.y, gyro.z);
    
    
    
    AHRSupdate(dt, gyro_act, accl_act, mag_act);
               
               
               

//	printf("A [%+7.2f, %+7.2f, %+7.2f]", phi, theta, psi);
 
    
    
//    printf("\n");

 }



















 
/* Set telemeter data from buffer */
void robot_parse_telemetry( unsigned char * buffer, unsigned int len )   // telemetry into buffer
{
    if (len != telemetry_size) 
    {
        alert(MODULE_NAME "Invalid telemetry length\n");
        return;
    }
    
    
    robot_imu(buffer);
    
                                                                                                
      // Save for transfer to client
    memcpy(robot.raw_telemetry, buffer, telemetry_size);
                 
                                                                                                            
    // Extract data for local use
    robot.analog_data[0] = get_int(buffer[0], buffer[1]);// / 2048.0;
    robot.analog_data[1] = get_int(buffer[2], buffer[3]);/// 2048.0;
    robot.analog_data[2] = get_int(buffer[4], buffer[5]);/// 2048.0;
    robot.analog_data[3] = get_int(buffer[6], buffer[7]);/// 2048.0;
             

    robot.serial_updates++;     
           
           
           
// Send the telmetry dfata to the client



    if (network_connected())
        robot_send_telemetry();
       
           
           
      
/*
    alert(MODULE_NAME "(%d) Analog:  %4.0f %4.0f %4.0f %4.0f\n", len,
                            robot.analog_data[0],   robot.analog_data[1],
                            robot.analog_data[2],   robot.analog_data[3]);                    

*/   
}
/* End of set_telemetry (buffer) */


/* Set new control data for Node IO board */
void robot_set_control(unsigned char * buffer, unsigned int len)
{
    robot.c_data[0] = buffer[0];
    robot.c_data[1] = buffer[1];
    robot.c_data[2] = buffer[2];
    robot.c_data[3] = buffer[3];
        
    robot.client_updates++; // we have new data, tell the failsafe
    
    robot_write_control();

       
}
/* End of robot_set_controller */


/* Send motor and servo data to the serial port */
void robot_write_control( void )
{
    unsigned char buff[10];


//printf("Robot Write control %x %x %x %x\n", robot.c_data[0],
// robot.c_data[1], robot.c_data[2], robot.c_data[3]);

  
    buff[0] = robot.c_data[0];
    buff[1] = robot.c_data[1];
    buff[2] = robot.c_data[2];
    buff[3] = robot.c_data[3];
    
#ifdef MDRIVER
   // alert(MODULE_NAME "MDRIVER PACKET\n");

    unsigned char packet[20];

    double l_motor = (double) (robot.c_data[0] - 0x80) - (double)(robot.c_data[1] - 0x80);
    double r_motor = (double) (robot.c_data[0] - 0x80) + (double)(robot.c_data[1] - 0x80);


    if (l_motor > 127) l_motor = 127;
    if (l_motor < -127) l_motor = -127;

    if (r_motor > 127) r_motor = 127;
    if (r_motor < -127) r_motor = -127;


    unsigned char l_motor_b = (unsigned char) l_motor;
    unsigned char r_motor_b = (unsigned char) r_motor;
    
    static unsigned char last_l_motor = 0;
    static unsigned char last_r_motor = 0;
    
     
    
    
//    if (l_motor_b == last_l_motor && r_motor_b == last_r_motor) return;


    last_l_motor = l_motor_b;
    last_r_motor = r_motor_b;




   // alert(MODULE_NAME "l: %f  r: %f\n", l_motor, r_motor);

    if (l_motor < 0)
    {
        packet[0] = 0x8A;
        packet[1] = (unsigned char ) fabs(l_motor);
    }else
    {
        packet[0] = 0x88;
        packet[1] = (unsigned char ) fabs(l_motor);
    }

    serial_send( packet, 2);
   // usleep(TIME_SECOND / UPDATE_RATE);
    
    
   // show_packet("l: ", "  ", packet, 2);
    
    

    if (r_motor < 0)
    {
        packet[0] = 0x8E;
        packet[1] = (unsigned char ) fabs(r_motor);
    }else
    {
        packet[0] = 0x8C;
        packet[1] = (unsigned char ) fabs(r_motor);
    }

    serial_send( packet, 2);
    
    //usleep(TIME_SECOND / UPDATE_RATE);
    
   // show_packet("r: ", "\n", packet, 2);
    
    
    
#else
         
    serial_write_packet(COMM_NI_CTRL, buff, COMM_NI_CTRL_LEN);  

#endif
    
    
    
    
}
/* End of send_control_packet */


/* Reset the motor controller to the default positions */
void robot_reset( void )
{
    alert(MODULE_NAME "Reset\n");

    robot.c_data[0] = 0x80;
    robot.c_data[1] = 0x80;
    robot.c_data[2] = 0x80;
    robot.c_data[3] = 0x80;

    robot.client_updates = 0;
    robot.serial_updates = 0;
}
/* End of reset_controller */





/* Controller thread */
void *robot_thread( void *threadid )
{
    alert(MODULE_NAME "Thread\n");

    while(!program_done) 
    {
        usleep(TIME_SECOND / UPDATE_RATE);
  
          
#ifndef MDRIVER                  
                                  
        if (!serial_flushing())
        {
            robot_write_control();
                       
            unsigned char buff[telemetry_size + 1]; // Leave roomm for packet type
                       
            unsigned int ret;



            ret = serial_read_packet(buff, telemetry_size + 1);

            if (ret == 0)
            {
                if (buff[0] == COMM_IN_TEL)
                    robot_parse_telemetry(buff+1, telemetry_size); // Strip packet type
                else
                    alert(MODULE_NAME "robot_thread invalid packet type: %d\n", buff[0]);
            }
            
//            usleep(250000);            
            
            
        }
#endif


            
    }

    pthread_exit(NULL); // exit thread
}
/* End of controller_thread */


/* Start Controller */
void robot_start( void )
{
    if (ROBOT_ENABLE == 0) return;

    alert(MODULE_NAME "Start\n");

    serial_start();
    
    network_start();
    
    failsafe_start();
    
    robot_reset();

    pthread_create(&robot.robot_t, NULL, robot_thread, NULL);
}
/* End of start_controller */


/* Stop controller */
void robot_stop( void )
{
    if (ROBOT_ENABLE == 0) return;
    
    alert(MODULE_NAME "Stop\n");

    serial_stop();
}
/* End of controller_stop */






