/*
    NullBot Robot Platform - Server - 2005-2011 Jason Hunt
    
    GNU GENERAL PUBLIC LICENSE Version 2, June 1991
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: nulluser@gmail.com
    
    File: serial.c
*/


#include <sys/ioctl.h>
#include <linux/i2c-dev.h>

#include <stdio.h>
#include <memory.h>
#include <termios.h> 
#include <fcntl.h> 
#include <unistd.h>
#include <pthread.h>
#include <fcntl.h>
#include <stdlib.h>

#include "../netcomm.h"
#include "utility.h"
#include "serial.h"
#include "server.h"
#include "robot.h"

#define MODULE_NAME "[Serial] "

//#define USE_I2C
//#define SERIAL_DEBUG

char serial_device[MAX_STR];

serial_type serial;


/* Return true iof the serial port is currently flushing */
int serial_flushing( void )
{
    return serial.flush > 0;
}
/* End of serial_flushing */


/* Reset serial structure */
void reset_serial( void )
{
    serial.state = COMM_PACKET_WAIT;
    
    serial.b_index = 0;
    serial.packet_size = 0;
    
    serial.flush = 0;
        
    serial.log = 1;
}
/* End of reset serial */


/* Flush serial line */
void serial_flush (void )
{
    // We need to flush the line
    alert(MODULE_NAME "Force controller reset\n");
    serial.flush = FLUSH_PERIOD;            // Set ignore count
        
    unsigned char buff[MAX_RX_PACKET ];
    
    alert(MODULE_NAME "Flushing serial\n");
            
    read(serial.serial_fd, buff, 10);
}
/* End of serial_flush */


/* Validate a serial packet */
int validate_packet( unsigned char *bf, unsigned int len )
{
    int c = get_checksum(bf, len-1);
    
    if (c != bf[len-1])
    {
        alert(MODULE_NAME "Invalid packet checksum actual: %02x  Expected: %02x\n", bf[len-1], c);
        //        show_buffer("Buffer: ", buff, len, 0);
        
        return(1);
    }
    
    return(0);
}
/* End of validate packet */


/* Process a serial packet */
/*int process_packet( unsigned char *bf, unsigned int len )
{
    if(serial.log)
        log_packet("Process: ", bf, len);

    if (validate_packet(bf, len))
    {
        // Force the controller to timeout and flush remaining
        alert(MODULE_NAME "Unable to validate packet\n");

        show_packet("Data: ", bf, len);

        serial_flush();
      

        return(1);
    }
    else
    {
        unsigned char device_id = bf[1];


//        alert(MODULE_NAME "Data for deviceid: %d", device_id);
        


        // dispatch the packet
        if (device_id == DEVICE_CONT+1)
        {
           // telemetry into buffer
            robot_parse_telemetry(bf + HEADER_SIZE, len - HEADER_SIZE  - FOOTER_SIZE);
       }
        else  
        if (device_id == DEVICE_CONT)
        {
            show_packet("Cont data: ", bf, len);

        } else
             
          
        if (device_id == DEVICE_DEBUG)
        {
            bf[len - FOOTER_SIZE] = 0;
                
            alert(MODULE_NAME "%s\n", (char *)(bf + HEADER_SIZE));
            
        }
        else
            alert(MODULE_NAME "Data for invalid device: %d\n", device_id);
            
    }
    
    return(0);
}*/
/* End of process_packet */

    
/* Process a serial byte */    
/*int process_byte( unsigned char b )
{
    //alert("%2x \n", b);


    // Ignore byte in flush mode
    if (serial.flush != 0)
    {
        log_write("Flush Ignore: %x\n", b);
        return(1);
    }



    // Decode current state
    if (serial.state == COMM_PACKET_WAIT)      // Waiting for packet start
    {
        serial.b_index = 0;

        serial.packet_size = b;
     
        serial.rx_buff[serial.b_index++] = b;
        
        if (serial.packet_size < SERIAL_MIN_PACKET )
        {
            alert(MODULE_NAME "Invalid packet size (%d) \n", serial.packet_size);
        
            serial.state = COMM_PACKET_WAIT;   
            
            serial.flush = FLUSH_PERIOD;
           
            
            return(1);                 
        }
        serial.state = COMM_PACKET_DATA;                    
    
    } else

    if (serial.state == COMM_PACKET_DATA)      // Getting packet data
    {
        if (serial.b_index >= COMM_RX_MAX)
        {
            alert(MODULE_NAME "Process byte buffer size exceeded\n");

            serial.state = COMM_PACKET_WAIT;
            
            serial.flush = FLUSH_PERIOD;

            return(1);
        }
    
        serial.rx_buff[serial.b_index++] = b;
    
        // We have the entire packer, process it
        if (serial.b_index == serial.packet_size)
        {
            serial.state = COMM_PACKET_WAIT;    

            if (process_packet( serial.rx_buff, serial.packet_size ))
            return(1);

        }
    }
    return(0);
}*/

/* End of process_byte */


/* Send a serial byte */
int serial_send( unsigned char *bf, unsigned int len )
{
    if (serial.flush > 0) 
        return 1;


    #ifdef SERIAL_DEBUG

    show_packet("Serial tx: ", "\n", bf, len);
  //  if (serial.log) 
        log_packet("Serial send", bf, len);
    #endif

    write(serial.serial_fd, bf, len);
    
    fsync(serial.serial_fd);     
    
    return 0;
    
}
/* End of serial_send */


/* Preform a transfer with the controller */
int serial_write_packet( unsigned char packet_type, 
                         unsigned char *send_buff, 
                         int send_bytes )
{
    // Do not write during a flush
    if (serial.flush > 0) 
        return 1;

    if (send_bytes <= 0) 
    {
        alert(MODULE_NAME "Attempt to send zero bytes");
        return(0);
    }
    
    unsigned char data[2] = {send_bytes + 3, packet_type};
    
    // Send packet header
    if (serial_send (data, 2) == -1) return(1);

    // Send packet data
    if (serial_send (send_buff, send_bytes) == -1) return(1);

    // Set checksum
    data[0] =  get_checksum(send_buff, send_bytes) + 
               get_checksum(data, 2) ;

    // Send packet footer
    if (serial_send (data, 1) == -1) return(1);
              
    fsync(serial.serial_fd);              
              
              
    return(0);
}
/* End of do_serial_packet */
  
  
/* Read a packet from the serial device */
int serial_read_packet(unsigned char *data, unsigned int len)
{
    int ret;
    unsigned int packet_len = len + 2;  // One byte for size, one for checksum

    // Local buffer, we need to read more than the requested length
    // This could be better
    unsigned char *rx_buff = (unsigned char *) malloc(packet_len);
    
    //#ifdef USEI2C
    
    unsigned int i;
    ret = 0;
    
    for (i = 0; i < packet_len; i++)
    {
        if (read(serial.serial_fd, rx_buff + i, 1) == 1)
            ret++;
        else
            break;
       
                
      //  usleep(10000);
    }
    
    //#else
        // Block read does not work on I2C
      //  ret =  read(serial.serial_fd, rx_buff, packet_len);
    //#endif
  
    // Make sure we got the expected number of bytes  
    if (ret != packet_len)
    {
        alert(MODULE_NAME "serial_read_packet - Read Len Invalid Actual: %d Expected:%d\n",
                          ret, packet_len);
                          
        if (ret > 0)                          
            show_packet("Serial rx: ", "\n", rx_buff, ret);
               
        free(rx_buff);
        serial_flush();
        return(1);
    }
        
    // Check the packet
    if (validate_packet( rx_buff, packet_len ))
    {
        alert(MODULE_NAME "serial_read_packet - Invalid Packet\n");

        show_packet("Serial rx: ", "\n", rx_buff, packet_len);

        free(rx_buff);
        serial_flush();
        return(1);
    
    }
    
    
//     show_packet("Serial rx: ", rx_buff, packet_len);
    
    
    
    // Copy over the data portion 
    // Strip checksum and packet size
    memcpy(data, rx_buff + 1, len);

    free(rx_buff);

    return(0);
}

  

/* Reading thread */
void *monitor_thread( void *threadid )
{
    alert(MODULE_NAME "Monitor Thread\n");

    while(!program_done) 
    {
        usleep(100000);
        
        if (serial.flush > 0)
        {
            //printf("Serial Flush: %d\n", serial.flush);

            serial.flush--;
            
            if (serial.flush == 0)
                alert(MODULE_NAME "Exit Flush\n");
        }
    }

    alert(MODULE_NAME "Monitor thread exit\n");
        
    pthread_exit(NULL); // exit thread 
}
/* End of read_thread */



/* Reading thread */
// Bug: this will not exit when the program stops 
void *read_thread( void *threadid )
{
//    unsigned int read_count;
//    unsigned char bf[10];

    alert(MODULE_NAME  "Read Thread\n");

    while(!program_done) 
    {
        usleep(500000);
    
    /*
       if (serial.flush == 0)
       {
   
          //  alert(MODULE_NAME "Read start");

          //  read_count =  read(serial.serial_fd, bf, 1);
                   
           // alert(MODULE_NAME "Read end, count %d", read_count);


//        log("(%d)  R Byte: %02x ", read_count , bf[0]);
                    
        if (read_count > 0)
            process_byte(bf[0]);

        #ifdef USE_I2C 
            usleep(I2C_BYTE_PERIOD);
        #endif    

        }
        else
            usleep(100000);

        // printf("Serial Flush: %d\n", serial.flush);   
            
        //if (read_count == 0)
         //   log_write("Serial timeout\n");
         */
         
            
    }

    alert(MODULE_NAME "Read thread exit\n");
        
    pthread_exit(NULL); // exit thread 
}
/* End of read_thread */


/* Start the serial system */
void serial_start( void )
{
    alert(MODULE_NAME "Start\n");
    
    serial.serial_fd = -1;
    
    #ifdef USE_I2C

//        int adapter_nr = 0; /* probably dynamically determined */
//        char filename[20];
//        snprintf(filename, 19, "/dev/i2c-%d", adapter_nr);

        serial.serial_fd = open(serial_device, O_RDWR);
    
        alert(MODULE_NAME "Using I2C on: %s\n  Addr: %x", filename, I2C_ADDR);

        if (serial.serial_fd < 0) 
        {
            alert(MODULE_NAME "Unable to open I2C\n");
            return;
        }

        int addr = I2C_ADDR; /* The I2C address */
	
        if (ioctl(serial.serial_fd, I2C_SLAVE, addr) < 0) 
        {
            alert(MODULE_NAME "Unable to set slave address\n");
            close(serial.serial_fd);
            
            return;
        }
    
    

  
    
   
    #else // Regular Serial

        alert(MODULE_NAME "Using RS232 on: %s\n", serial_device);
    
    
        serial.serial_fd = open(serial_device, O_RDWR | O_NOCTTY | O_NDELAY);

        if (serial.serial_fd == -1)
        {
            alert(MODULE_NAME "Unable to open serial port\n");
            return;
        }
        else
            fcntl(serial.serial_fd, F_SETFL, 0);

        struct termios options;

        tcgetattr(serial.serial_fd, &serial.save_options); // Save old options
    
        bzero(&options, sizeof(options));

        options.c_cflag |= BAUDRATE | CLOCAL | CREAD | CS8;
    
        options.c_lflag = 0;
    
        options.c_cc[VTIME]    = 5;  
        options.c_cc[VMIN]     = 0;  
        
        tcflush(serial.serial_fd, TCIFLUSH);
        tcsetattr(serial.serial_fd, TCSANOW, &options);

    #endif

    reset_serial();  
  
    serial_flush ();

    pthread_create(&serial.read_t, NULL, read_thread, NULL);
    pthread_create(&serial.monitor_t, NULL, monitor_thread, NULL);
}
/* End of start_serial */


/* Stop the serial system */
void serial_stop( void )
{
    alert(MODULE_NAME "Stop\n");

    #ifdef USE_I2c

    #else
    
    printf("fd: %d\n", serial.serial_fd);
    
        if (serial.serial_fd != -1)
            tcsetattr(serial.serial_fd, TCSANOW, &serial.save_options);      
    #endif

    close(serial.serial_fd);
}
/* End of serial_stop */




