/*
	NullBot Robot Platform - NodeIO - 2005-2011 Jason Hunt

	Contact: nulluser@gmail.com
    
	GNU GENERAL PUBLIC LICENSE Version 2, June 1991
	
	This program is free software; you can redistribute it and/or
    	modify it under the terms of the GNU General Public License
    	as published by the Free Software Foundation; either version 2
    	of the License, or (at your option) any later version.

    	This program is distributed in the hope that it will be useful,
    	but WITHOUT ANY WARRANTY; without even the implied warranty of
    	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    	GNU General Public License for more details.

    	You should have received a copy of the GNU General Public License
    	along with this program; if not, write to the Free Software
    	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

	File: analog.c
*/


#include "core.h"

unsigned int analog[NUM_ANALOG];
unsigned int analog_raw[NUM_ANALOG];
unsigned int samples[NUM_ANALOG][NUM_SAMPLES];

unsigned long sample_count = 0;
unsigned long sample_rate = 0;



unsigned char sig_analog = 0;


/* ADC Interrupt */
void __attribute__((__interrupt__, auto_psv)) _ADCInterrupt (void)
{
	unsigned int *p = (unsigned int *) &ADCBUF0;

	unsigned char i;

	// Grab all Samples
	for (i = 0; i < NUM_ANALOG; i++)
		analog_raw[i] = *p++;
	
	sig_analog = 1;

	ADCON1bits.ASAM = 0;	// cancel auto sample for processing

	IFS0bits.ADIF = 0;
}
/* End of _ADCInterrupt */



/* Update analog samples from raw values */
void analog_task( void )
{
	if (sig_analog == 0) return;

	sig_analog = 0;


	unsigned char i,j;


	//return;

	static unsigned char init = 0;


	// Furst run
	if (init)
	{
		// Fill the sample buffer with the current values
		for (i = 0; i < NUM_ANALOG; i++)
		{
			for (j = 0; j < NUM_SAMPLES; j++)
				samples[i][j] = analog_raw[i];
		}

		init = 0;
	
	} 

	// Grab Samples
	for (i = 0; i < NUM_ANALOG; i++)
	{
		// Compute average for this channel
		// Should be replaced with an IIR
		unsigned long sum = 0;
		
		for (j = 1; j < NUM_SAMPLES; j++)
		{
			sum +=  samples[i][j];	
			samples[i][j-1] = samples[i][j];
		}

		sum += analog_raw[i];
		samples[i][NUM_SAMPLES - 1] = analog_raw[i];
				
		analog[i] = sum / NUM_SAMPLES;

	}

	sample_count++;


	// Setup Capture for all samples	
	ADCON1bits.ASAM = 1;
}
/* End of update analog */


/* Analog timer handler */
void analog_timer ( void )
{
	//ADCON1bits.SAMP = 1;
	static unsigned int sec_count = 0;

	// Update sample rate every second
	if (++sec_count > 100)
	{
		sample_rate = sample_count;
		sample_count = 0;
		sec_count=0;
	}






}
/* End of analog_timer */


/* Setup ADC */
void analog_init ( void )
{
	//TRISB = 0x1000;

	TRISB =  0b0001110100000000; // RB11, RB12 = Inputs, others: Outputs
	ADPCFG = 0b1110001011111111; // RB11, RB12 = analog

	ADCSSL =  0b0001110100000000;


	ADCON1bits.SSRC = 7;

	ADCON2bits.SMPI = NUM_ANALOG-1; // samples per interrupt

	ADCON2bits.CSCNA = 1;

	ADCON3bits.SAMC = 31;
	ADCON3bits.ADCS = 63;//13;//6;     

	// Fcy = 20000000
	// Tcy = 50 nS
	// ADCS = 31
	// Tad = 800nS
	// 

	IEC0bits.ADIE = 1;

	ADCON1bits.ADON = 1; // turn ADC ON

//	ADCON1bits.SAMP = 1;

	ADCON1bits.ASAM = 1;

	timer_register(analog_timer);
	task_register(analog_task);
}
/* End of analog_init */


