/*
	NullBot Robot Platform - NodeIO - 2005-2011 Jason Hunt

	Contact: nulluser@gmail.com
    
	GNU GENERAL PUBLIC LICENSE Version 2, June 1991
	
	This program is free software; you can redistribute it and/or
    	modify it under the terms of the GNU General Public License
    	as published by the Free Software Foundation; either version 2
    	of the License, or (at your option) any later version.

    	This program is distributed in the hope that it will be useful,
    	but WITHOUT ANY WARRANTY; without even the implied warranty of
    	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    	GNU General Public License for more details.

    	You should have received a copy of the GNU General Public License
    	along with this program; if not, write to the Free Software
    	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "core.h"



/* Servo */
static unsigned char servo_mode = 0;			// Current state of the servo interrupt system
static unsigned char cur_servo = 0;			// Current servo 
static unsigned char servo_pos[NUM_SERVO];
static char servo_trim[NUM_SERVO];




/* IMU */
/*static int rate_z_offset = 0;
static int accl_x_offset = 0;
static int accl_y_offset = 0;


static int rate_z_diff =0;
static int accl_x_diff =0;
static int accl_y_diff=0;


static long int angle_z  = 0;

static int vel_x  = 0;
static int vel_y  = 0;

static int pos_x  = 0;
static int pos_y  = 0;*/

//static unsigned int sample_time = 0;
static unsigned int is_moving = 0;


/* Telemetry */

unsigned char tel_packet[COMM_IN_TEL_LEN + 3];
unsigned int tel_index = 0;


/* Debugging */
unsigned char blink_enable = 1;





/* Update the servos */
void update_servos( void )
{
	if (servo_mode == 0) // setupup for base pos wait
	{
		servo_mode = 1; 					   // next trigger we will setup for base delay

		if (IDLE_SERVOS)
		{
			if (cur_servo == 0 && servo_pos[0] != 0x80) _LATF0 = 1; else
			if (cur_servo == 1 && servo_pos[1] != 0x80) _LATF1 = 1; else
			if (cur_servo == 2 && servo_pos[2] != 0x80) _LATF6 = 1; else
			if (cur_servo == 3 && servo_pos[3] != 0x80) _LATD0 = 1; 
		} else
		{
			if (cur_servo == 0 ) _LATF0 = 1; else
			if (cur_servo == 1 ) _LATF1 = 1; else
			if (cur_servo == 2 ) _LATF6 = 1; else
			if (cur_servo == 3 ) _LATD0 = 1; 
		}

		PR2 = 2550 + (unsigned int)(servo_pos[cur_servo] + servo_trim[cur_servo])* 10;

	} else
	if (servo_mode == 1)					// setupfor next servo
	{
		servo_mode = 0;						// reset back to base mode

		if (cur_servo == 0) _LATF0 = 0; else
		if (cur_servo == 1) _LATF1 = 0; else
		if (cur_servo == 2) _LATF6 = 0; else
		if (cur_servo == 3) _LATD0 = 0; 

		cur_servo++; // next servo
		
		// Setup longer delay to fill the 20 ms window, 
		// we need to burn 11ms on average
		// 5Mhz clock, 0.0512 MS per increment at 1:256 prescaler
		// 11ms / 0.0512 ms = 215 cycles to burn, start time at 40

		if (cur_servo == NUM_SERVO)
		{
			cur_servo = 0;	// reset back the first servo
			PR2 = 45000;
		}
	}
}
/* End of void update_servoes */


/* Send controller telemetry */
void controller_telemetry( void )
{
	tel_packet[0] = COMM_IN_TEL_LEN + 3;	// Total packet size
	tel_packet[1] = COMM_IN_TEL;		// Packet Type

	tel_packet[2] = imu_data[0];
	tel_packet[3] = imu_data[1];
	tel_packet[4] = imu_data[2];
	tel_packet[5] = imu_data[3];
	tel_packet[6] = imu_data[4];
	tel_packet[7] = imu_data[5];

	tel_packet[8] = imu_data[6];
	tel_packet[9] = imu_data[7];
	tel_packet[10] = imu_data[8];
	tel_packet[11] = imu_data[9];
	tel_packet[12] = imu_data[10];
	tel_packet[13] = imu_data[11];

	tel_packet[14] = imu_data[12];
	tel_packet[15] = imu_data[13];
	tel_packet[16] = imu_data[14];
	tel_packet[17] = imu_data[15];
	tel_packet[18] = imu_data[16];
	tel_packet[19] = imu_data[17];



	tel_packet[20] = fault_data;
	tel_packet[21] = fault;

	tel_packet[22] = get_checksum(tel_packet, COMM_IN_TEL_LEN + 2);

	write_packet(tel_packet, COMM_IN_TEL_LEN + 3);

//	for (i = 0; i < COMM_IN_TEL_LEN + 3; i++)
		//serial_tx(0, tel_packet[i]);

//	msg_telemetry = 0;


}
/* End of controller_telemetry */


/* Deal with control packet */
void controller_packet( unsigned char *buff )
{

	// Send back telemetry
	// Not not do this for I2C because it will be polled
	#ifndef I2CHOST
	
		controller_telemetry();
	#endif


	// EXtract the control data
	servo_pos[0] = buff[0];
	servo_pos[1] = buff[1];
	servo_pos[2] = buff[2];
	servo_pos[3] = buff[3];
	



	is_moving = 0;

	if (servo_pos[0] != SERVO_CENTER ||
	    servo_pos[1] != SERVO_CENTER ||
	    servo_pos[2] != SERVO_CENTER ||
	    servo_pos[3] != SERVO_CENTER)
		is_moving = 1;

	// We got an update, clear the timeout flag

	comm_clear_timeout();

}
/* End of controller_packet */


/* Reset the controller */
void controller_reset( void )
{
	unsigned int i;

	for (i = 0; i < NUM_SERVO; i++)
		servo_pos[i] = SERVO_CENTER;

	tel_index = 0;

	//count = 0;
}
/* End of void controller_reset */


/* Update the IMU data */
void update_IMU( void )
{
/*	static int imu_count = 0;

//	if (imu_count++ > 1000)
	//{
		imu_count = 0;


	sample_time = TMR3;

	TMR3 = 0;

	rate_z_diff = analog[1] - rate_z_offset;
	accl_x_diff = analog[2] - accl_x_offset;
	accl_y_diff = analog[3] - accl_y_offset;

	static int init_count = 9;
	static char init = 0;


	if (init == 0)
	{
		// Average the samples before setting offsets
		if (init_count++ > 100)
		{
			rate_z_offset = rate_z_diff;
			accl_x_offset = accl_x_diff; 
			accl_y_offset = accl_y_diff;

			init = 1;
		} 
		return;
	}	

	//int deadband1 = 10;
	//int deadband2 = 40;



//	if (ABS(rate_z_diff) >= deadband1) 
	if (is_moving)
	{
		angle_z += rate_z_diff;
	}

//	if (ABS(accl_x_diff) >= deadband2 || 
	    //ABS(accl_x_diff) >= deadband2)
	
	if (is_moving)
	{
		vel_x += accl_x_diff;
		vel_y += accl_y_diff;

		pos_x += vel_x;
		pos_y += vel_y;
	} else
	{

		vel_x = 0;
		vel_y = 0;

	}
*/


}
/* End of update IMU */


/* update controller */
void controller_task( void )
{
	update_IMU();

}
/* End of controller process */


/* Update controller */
void controller_timer ( void )
{


}
/* End of controller_timer */


/* Setup controller data */
void controller_init ( void )
{
	controller_reset();

/*	servo_trim[0] = 6;
	servo_trim[1] = 0;
	servo_trim[2] = -6;
	servo_trim[3] = 6;*/


	servo_trim[0] = 0;
	servo_trim[1] = 0;
	servo_trim[2] = 0;
	servo_trim[3] = 0;


	timer_register(controller_timer);
	task_register(controller_task);

}
/* End of controller_init */


