/*
	NullBot Robot Platform - NodeIO - 2005-2011 Jason Hunt

	Contact: nulluser@gmail.com
    
	GNU GENERAL PUBLIC LICENSE Version 2, June 1991
	
	This program is free software; you can redistribute it and/or
    	modify it under the terms of the GNU General Public License
    	as published by the Free Software Foundation; either version 2
    	of the License, or (at your option) any later version.

    	This program is distributed in the hope that it will be useful,
    	but WITHOUT ANY WARRANTY; without even the implied warranty of
    	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    	GNU General Public License for more details.

    	You should have received a copy of the GNU General Public License
    	along with this program; if not, write to the Free Software
    	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

	File: i2c.c
*/



#include <i2c.h>

#include "core.h"


#ifdef I2C_ENABLE

void __attribute__((__interrupt__, auto_psv)) _MI2CInterrupt (void)
{
	IFS0bits.MI2CIF = 0;
}

void __attribute__((__interrupt__, auto_psv)) _SI2CInterrupt (void)
{
	IFS0bits.SI2CIF = 0;
}



/* Start I2C Transaction */
int i2c_start(unsigned char address)
{
	IdleI2C();
	StartI2C();

	while(I2CCONbits.SEN ); // Wait for start complete

	IdleI2C();

	MasterWriteI2C(address); // Write slave address

	while(I2CSTATbits.TBF); // Wait until complete

	IdleI2C();

	while(I2CSTATbits.ACKSTAT); // Wait for ACK

	IdleI2C();

	return I2C_SUCCESS;
}
/* End of I2C Start */


/* Send restart condition */
void i2c_restart( void )
{
	RestartI2C();

	while (I2CCONbits.RSEN); // Wait for restart

	IdleI2C();
}
/* End of restart */


/* End of I2C Stop */
void i2c_stop()
{
	StopI2C();

	while(I2CCONbits.PEN); // Wait for stop

	IdleI2C();
}
/* End of i2c_start */



/* Read I2C byte */
unsigned char i2c_read( void)
{
	unsigned char c;

	c = MasterReadI2C();
	
	IdleI2C();

	return c;
}
/* End of i2c_read */


/* ACK i2c message */
void i2c_ack ( void )
{
	AckI2C();
	
	while ( I2CCONbits.ACKEN );

	IdleI2C();
}
/* End of i2c_ack */


/* NACK I2C Mesasge */
void i2c_nack ( void )
{
	NotAckI2C();
	
	while ( I2CCONbits.ACKEN );

	IdleI2C();
}
/* End of i2c_nack */


/* Read i2c multiple words */
unsigned char i2c_readbytes(unsigned char *data, unsigned int len)
{
	int i;

	for (i = 0; i < len; i++)
	{
		// Get byte from slave
		data[i] = i2c_read();
	
		// NACK last byte, otherwise ACK
		if (i < len - 1)
			i2c_ack();
		else
			i2c_nack();
	}
	
	return I2C_SUCCESS;
}
/* End iof i2c_readbytes */



/* Write i2c_byte */
void i2c_write( unsigned char v)
{
	MasterWriteI2C(v);

	IdleI2C();

	while(I2CSTATbits.ACKSTAT); // Wait for ACK

	IdleI2C();
}
/* End of i2c_write */



/* Write I2C Device register */
unsigned char i2c_write_reg(unsigned char address, unsigned char reg, unsigned char value)
{
	i2c_start(address);
	i2c_write(reg);
	i2c_write(value);
	i2c_stop();
	
	return 0;
}
/* End of i2c_write_reg */

#endif

/* Init the I2C module */
void i2c_init( void )
{
  #ifdef I2C_ENABLE
	
	_LATF2 = 0;   //  From dsPIC30F4013  eratta

	unsigned int config2, config1;

	// Compute BRG
	config2 = (FCY / I2C_BAUD - I2C_PGD * FCY) - 1;  // 181

  	// 7 bit address mode
   	config1 = (I2C_ON & I2C_IDLE_CON & I2C_CLK_HLD &
               I2C_IPMI_DIS & I2C_7BIT_ADD &
               I2C_SLW_DIS & I2C_SM_DIS &
               I2C_GCALL_DIS & I2C_STR_DIS &
               I2C_NACK & I2C_ACK_DIS & I2C_RCV_DIS &
               I2C_STOP_DIS & I2C_RESTART_DIS &
               I2C_START_DIS);

    OpenI2C(config1,config2);

    IFS0bits.SI2CIF = 0; // Make sure interrupts are disabled
    IEC0bits.SI2CIE = 0;

    IdleI2C();				// Make sure line is idle

	timer_delay(10);		// Allow devices to stabilize

  #endif
}
/* End of i2c_init */

