/*
	NullBot Robot Platform - NodeIO - 2005-2011 Jason Hunt

	Contact: nulluser@gmail.com
    
	GNU GENERAL PUBLIC LICENSE Version 2, June 1991
	
	This program is free software; you can redistribute it and/or
    	modify it under the terms of the GNU General Public License
    	as published by the Free Software Foundation; either version 2
    	of the License, or (at your option) any later version.

    	This program is distributed in the hope that it will be useful,
    	but WITHOUT ANY WARRANTY; without even the implied warranty of
    	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    	GNU General Public License for more details.

    	You should have received a copy of the GNU General Public License
    	along with this program; if not, write to the Free Software
    	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

	File: imu.c
*/

#include "core.h"


#ifdef IMU_ENABLE


unsigned char imu_data[18] = {0};
unsigned char imu_zero[18] = {0};


unsigned int i2c_count = 0;
unsigned char sig_imu = 0;				// Signal to capture IMU data



/* Get ACC Data */
void imu_get_acc( void )
{
	i2c_start(ACC_ADDRESS);

	i2c_write(LSM303_OUT_X_L_A | (1 << 7));

	i2c_restart();

	i2c_write(ACC_ADDRESS | I2C_READ);

	i2c_readbytes(imu_data, IMU_DATALEN);

	i2c_stop();
}
/* End of imu_get_acc */



/* Get MAG data */
void imu_get_mag( void )
{
	i2c_start(MAG_ADDRESS);

	i2c_write(LSM303_OUT_X_H_M);

	i2c_restart();

	i2c_write(MAG_ADDRESS | I2C_READ);

	i2c_readbytes(imu_data+IMU_DATALEN, IMU_DATALEN);

	i2c_stop();
}
/* End of imu_get_mag */


/* Get gyrp data */
void imu_get_gyro( void )
{
	i2c_start(GYRO_ADDRESS);

	i2c_write(L3G_OUT_X_L | (1 << 7));

	i2c_restart();

	i2c_write(GYRO_ADDRESS | I2C_READ);

	i2c_readbytes(imu_data+2*IMU_DATALEN, IMU_DATALEN);
	
	i2c_stop();
}
/* End of imu_get_gyro */


/* Update serial system */
// Call in interrupt, must return quickly
void imu_timer( void )
{
	sig_imu = 1;
}
/* End of serial_timer */


/* Update serial system */
void imu_task( void )
{
	static unsigned char init = 0;

	if (sig_imu)
	{
		sig_imu = 0;

		imu_get_acc();
		imu_get_mag();
		imu_get_gyro();
	}
}
/* End of serial_timer */

#endif 


/* Init the I2C module */
void imu_init( void )
{
  #ifdef IMU_ENABLE
	
	// Enable all axis, 100khz	
	i2c_write_reg(ACC_ADDRESS, LSM303_CTRL_REG1_A, 0x27);

	// Cont mode
	i2c_write_reg(MAG_ADDRESS, LSM303_MR_REG_M, 0x00);

	// Normal power, all axes
	i2c_write_reg(GYRO_ADDRESS, L3G_CTRL_REG1, 0x0F);

	// Setup Timer and Task
	timer_register(imu_timer);
	task_register(imu_task);

  #endif
}
/* End of i2c_init */

