/*
	NullBot Robot Platform - NodeIO - 2005-2011 Jason Hunt

	Contact: nulluser@gmail.com
    
	GNU GENERAL PUBLIC LICENSE Version 2, June 1991
	
	This program is free software; you can redistribute it and/or
    	modify it under the terms of the GNU General Public License
    	as published by the Free Software Foundation; either version 2
    	of the License, or (at your option) any later version.

    	This program is distributed in the hope that it will be useful,
    	but WITHOUT ANY WARRANTY; without even the implied warranty of
    	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    	GNU General Public License for more details.

    	You should have received a copy of the GNU General Public License
    	along with this program; if not, write to the Free Software
    	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

	File: main.c
*/


#include "core.h"

_FOSC(CSW_FSCM_OFF & XT_PLL8);	
_FWDT(WDT_OFF);
_FBORPOR(PBOR_OFF & BORV_27 & PWRT_16 & MCLR_EN);
_FGS(CODE_PROT_OFF);


void (*tasks[MAX_TASKS])(void);		// Task poll group
unsigned num_tasks = 0;

unsigned char fault = 0;
unsigned char fault_data = 0;



/* Register a task in the poll group */
void task_register ( void fn (void) )
{

	if (num_tasks >= MAX_TASKS-1) return;

	interrupt_disable() 

	tasks[num_tasks++] = fn;
	interrupt_enable() 
}
/* End of register task */


/* poll all tasks in poll group */
void task_poll ( void )
{
	unsigned int i;
	// Call all timer hooks
	for (i = 0; i < num_tasks; i++)
		(*tasks[i])();
}
/* End of poll tasks */


/* Do a short beep on the buzzer for debugging */
// Hard coded delay loops, can be used inside interrupts
void beep (void)
{
/*
	unsigned int i, j;

	for (i = 0; i < 40; i++)
	{
		buzzer = ~buzzer;

		for (j = 0; j < 3000; j++);
	}

	buzzer = 0;
*/
}
/* End of beep */


/* Setup the device */
void device_init ( void )
{
	// Set port directions
	TRISA = 0;
	TRISB = 0;
	TRISC = 0;
	TRISD = 0;
	
	TRISF = 0b00010100;

// f3 needs to be an output for serial

	_LATF0 = 0; 
	_LATF1 = 0; 
	_LATF6 = 0;
	_LATD0 = 0; 


	INTCON1bits.NSTDIS = 1;

}
/* End of device_init */


/* Main */
int main ( void )
{
	device_init();
	timer_init();
	comm_init();
	serial_init();
	i2c_init();
	imu_init();
	analog_init();
	controller_init();

	timer_delay(50);

	
	fault |= FAULT_OK;


	// Poll usermode tasks
	while (1)
	{
		task_poll();
	}

	return(0);
}
/* End of main */


