/*
	NullBot Robot Platform - NodeIO - 2005-2011 Jason Hunt

	Contact: nulluser@gmail.com
    
	GNU GENERAL PUBLIC LICENSE Version 2, June 1991
	
	This program is free software; you can redistribute it and/or
    	modify it under the terms of the GNU General Public License
    	as published by the Free Software Foundation; either version 2
    	of the License, or (at your option) any later version.

    	This program is distributed in the hope that it will be useful,
    	but WITHOUT ANY WARRANTY; without even the implied warranty of
    	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    	GNU General Public License for more details.

    	You should have received a copy of the GNU General Public License
    	along with this program; if not, write to the Free Software
    	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

	File: timer.c
*/

#include "core.h"

void (*timers[MAX_TIMERS])(void);		// Task poll group

unsigned num_timers = 0;
unsigned int delay_count = 0;

unsigned int timer_count = 0;

/* Delay in d is in seconds * 100 */
void timer_delay( unsigned int d )
{
	delay_count = 0;
	while(delay_count < d);
}
/* End of timer delay */



/* This will fire every 10 milliseconds */
void __attribute__((__interrupt__, auto_psv)) _T1Interrupt (void)
{
	IFS0bits.T1IF = 0;
	delay_count++;
	timer_count++;
	timer_poll();
}
/* End of  _T1Interrupt  */



/* Servo interrupt */
void __attribute__((__interrupt__, auto_psv)) _T2Interrupt (void)
{
	IFS0bits.T2IF = 0;
	update_servos();
}
/* End of  _T2Interrupt */



/* Register a timer in the poll group */
void timer_register ( void fn (void) )
{
	if (num_timers >= MAX_TIMERS-1) return;

	interrupt_disable() 

	timers[num_timers++] = fn;

	// We need reclaculate the timer interval for new number of tasks

	PR1 = FCY / 64 / TIMER1_FREQ / num_timers;


	interrupt_enable() 
}
/* End of register timer */


/* Poll Next timer in poll group */
// This will even out the load so we don't spend too much time in the timer interrupt
void timer_poll( void )
{
	static unsigned char cur_timer = 0;

	(*timers[cur_timer])();

	cur_timer++;

	if (cur_timer >= num_timers) 
		cur_timer = 0;
}
/* End of poll timers */



/* Setup the timer */
void timer_init ( void )
{
	// Setup timer for 1ms and enble interrupt
	T1CON = 0x20;			// Set prescaler to TCY / 64 
	PR1 = FCY / 64 / TIMER1_FREQ;
	T1CONbits.TON = 1;		// Enable timer
	IEC0bits.T1IE = 1;		// Enable interrupt

	// Setup timer for 1ms and enble interrupt
//	T1CON = 0x20;			// Set prescaler to TCY / 8
	T2CONbits.TCKPS = 0b01; // 8 prescale
	PR2 = FCY / 64 / TIMER1_FREQ;
	T2CONbits.TON = 1;		// Enable timer
	IEC0bits.T2IE = 1;		// Enable interrupt


	// Setup timer for 1ms and enble interrupt
//	T1CON = 0x20;			// Set prescaler to TCY / 8
	T3CONbits.TCKPS = 0b01; // 8 prescale
	//PR2 = FCY / 64 / TIMER1_FREQ;
	T3CONbits.TON = 1;		// Enable timer
	//IEC0bits.T2IE = 1;	


}
/* End of timer_init */



